from typing import List, Dict, Any, Tuple, Union
from datetime import datetime
import time
from csle_base.json_serializable import JSONSerializable
import csle_collector.constants.constants as constants


class DockerStats(JSONSerializable):
    """
    DTO class containing docker statistics
    """

    def __init__(self, pids: float = 0.0, timestamp: str = "", cpu_percent: float = 0.0, mem_current: float = 0.0,
                 mem_total: float = 0.0, mem_percent: float = 0.0, blk_read: float = 0.0, blk_write: float = 0.0,
                 net_rx: float = 0.0, net_tx: float = 0.0, container_name: str = "",
                 ip: Union[str, None] = None, ts: Union[None, float] = None):
        """
        Class constructor, creates a DockerStats object

        :param pids: the number of pids in the container
        :param timestamp: the timestamp that the stats were read
        :param cpu_percent: the CPU utilization percentage
        :param mem_current: the current memory usage
        :param mem_total: the memory limit
        :param mem_percent: the memory utilization
        :param blk_read: the number of read IO bytes
        :param blk_write: the number of written IO bytes
        :param net_rx: the number of receives network bytes
        :param net_tx: the number of sent network bytes
        :param container_name: the name of the container
        :param ip: the ip
        :param ts: the timestamp
        """
        self.pids = pids
        self.timestamp = timestamp
        self.cpu_percent = cpu_percent
        self.mem_current = mem_current
        self.mem_total = mem_total
        self.mem_percent = mem_percent
        self.blk_read = blk_read
        self.blk_write = blk_write
        self.net_rx = net_rx
        self.net_tx = net_tx
        self.container_name = container_name
        self.ip = ip
        self.ts = ts

    @staticmethod
    def from_dict(parsed_stats_dict: Dict[str, Any]) -> "DockerStats":
        """
        Parses a DockerStats object from a dict

        :param parsed_stats_dict: the dict to parse
        :return: the parsed DockerStats object
        """
        return DockerStats(pids=parsed_stats_dict[constants.DOCKER_STATS.PIDS],
                           timestamp=parsed_stats_dict[constants.DOCKER_STATS.TIMESTAMP],
                           cpu_percent=parsed_stats_dict[constants.DOCKER_STATS.CPU_PERCENT],
                           mem_current=parsed_stats_dict[constants.DOCKER_STATS.MEM_CURRENT],
                           mem_total=parsed_stats_dict[constants.DOCKER_STATS.MEM_TOTAL],
                           mem_percent=parsed_stats_dict[constants.DOCKER_STATS.MEM_PERCENT],
                           blk_read=parsed_stats_dict[constants.DOCKER_STATS.BLK_READ],
                           blk_write=parsed_stats_dict[constants.DOCKER_STATS.BLK_WRITE],
                           net_rx=parsed_stats_dict[constants.DOCKER_STATS.NET_RX],
                           net_tx=parsed_stats_dict[constants.DOCKER_STATS.NET_TX],
                           container_name=parsed_stats_dict[constants.DOCKER_STATS.CONTAINER_NAME])

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation
        
        :return: a dict representation of the object
        """
        return {
            constants.DOCKER_STATS.PIDS: self.pids,
            constants.DOCKER_STATS.TIMESTAMP: self.timestamp,
            constants.DOCKER_STATS.CPU_PERCENT: self.cpu_percent,
            constants.DOCKER_STATS.MEM_CURRENT: self.mem_current,
            constants.DOCKER_STATS.MEM_TOTAL: self.mem_total,
            constants.DOCKER_STATS.MEM_PERCENT: self.mem_percent,
            constants.DOCKER_STATS.BLK_READ: self.blk_read,
            constants.DOCKER_STATS.BLK_WRITE: self.blk_write,
            constants.DOCKER_STATS.NET_RX: self.net_rx,
            constants.DOCKER_STATS.NET_TX: self.net_tx,
            constants.DOCKER_STATS.CONTAINER_NAME: self.container_name
        }

    def __str__(self):
        """
        :return: a string representation of the object
        """
        return f"pids: {self.pids}, timestamp: {self.timestamp}, cpu_percent: {self.cpu_percent}%, " \
               f"mem_current: {self.mem_current}MB, mem_total: {self.mem_total}MB, mem_percent: {self.mem_percent}%," \
               f"blk_read: {self.blk_read}MB, blk_write: {self.blk_write}MB, net_rx: {self.net_rx}MB, " \
               f"net_tx: {self.net_tx}MB, container_name: {self.container_name}"

    @staticmethod
    def compute_averages(stats_list: List["DockerStats"]) -> "DockerStats":
        """
        Compute average stats from a list

        :param stats_list: the list to use to compute averages
        :return: a new DockerStats object with the averages
        """
        if len(stats_list) == 0:
            now = datetime.now()
            ts = now.strftime("%m/%d/%Y, %H:%M:%S")
            return DockerStats(pids=0.0, timestamp=ts, cpu_percent=0.0, mem_current=0.0, mem_total=0.0, mem_percent=0.0,
                               blk_read=0.0, blk_write=0.0, net_rx=0.0, net_tx=0.0, container_name="-")
        sum_pids = 0.0
        sum_cpu_percent = 0.0
        sum_mem_current = 0.0
        sum_mem_total = 0.0
        sum_mem_percent = 0.0
        sum_blk_read = 0.0
        sum_blk_write = 0.0
        sum_net_rx = 0.0
        sum_net_tx = 0.0
        for i in range(len(stats_list)):
            sum_pids = sum_pids + stats_list[i].pids
            sum_cpu_percent = sum_cpu_percent + stats_list[i].cpu_percent
            sum_mem_current = sum_mem_current + stats_list[i].mem_current
            sum_mem_total = sum_mem_total + stats_list[i].mem_total
            sum_mem_percent = sum_mem_percent + stats_list[i].mem_percent
            sum_blk_read = sum_blk_read + stats_list[i].blk_read
            sum_blk_write = sum_blk_write + stats_list[i].blk_write
            sum_net_rx = sum_net_rx + stats_list[i].net_rx
            sum_net_tx = sum_net_tx + stats_list[i].net_tx

        avg_pids = float("{:.1f}".format(float(sum_pids / len(stats_list))))
        avg_cpu_percent = float("{:.1f}".format(float(sum_cpu_percent / len(stats_list))))
        avg_mem_current = float("{:.1f}".format(float(sum_mem_current / len(stats_list))))
        avg_mem_total = float("{:.1f}".format(float(sum_mem_total / len(stats_list))))
        avg_mem_percent = float("{:.1f}".format(float(sum_mem_percent / len(stats_list))))
        avg_blk_read = float("{:.1f}".format(float(sum_blk_read / len(stats_list))))
        avg_blk_write = float("{:.1f}".format(float(sum_blk_write / len(stats_list))))
        avg_net_rx = float("{:.1f}".format(float(sum_net_rx / len(stats_list))))
        avg_net_tx = float("{:.1f}".format(float(sum_net_tx / len(stats_list))))

        ts = stats_list[0].timestamp
        container_name = stats_list[0].container_name

        return DockerStats(pids=avg_pids, timestamp=ts, cpu_percent=avg_cpu_percent, mem_current=avg_mem_current,
                           mem_total=avg_mem_total, mem_percent=avg_mem_percent,
                           blk_read=avg_blk_read, blk_write=avg_blk_write, net_rx=avg_net_rx, net_tx=avg_net_tx,
                           container_name=container_name)

    @staticmethod
    def from_kafka_record(record: str) -> "DockerStats":
        """
        Converts a kafka record to a DTO

        :param record: the recordto convert
        :return: the created DTO
        """
        parts = record.split(",")
        obj = DockerStats(ts=float(parts[0]), ip=parts[1], cpu_percent=float(parts[2]), mem_current=float(parts[3]),
                          mem_total=float(parts[4]), mem_percent=float(parts[5]), blk_read=float(parts[6]),
                          blk_write=float(parts[7]), net_rx=float(parts[8]), net_tx=float(parts[9]), container_name="",
                          pids=int(round(float(parts[10]))), timestamp=str(parts[0]))
        return obj

    def update_with_kafka_record(self, record: str) -> None:
        """
        Updates the DTO with a given kafka record

        :param record: the record to update with
        :return: None
        """
        parts = record.split(",")
        self.ts = float(parts[0])
        self.ip = parts[1]
        self.cpu_percent = float(parts[2])
        self.mem_current = float(parts[3])
        self.mem_total = float(parts[4])
        self.mem_percent = float(parts[5])
        self.blk_read = float(parts[6])
        self.blk_write = float(parts[7])
        self.net_rx = float(parts[8])
        self.net_tx = float(parts[9])
        self.container_name = ""
        self.pids = int(round(float(parts[10])))
        self.timestamp = str(parts[0])

    def update_with_kafka_record_ip(self, record: str, ip: str) -> None:
        """
        Updates the DTO with a given kafka record

        :param record: the record to update with
        :param ip: the ip of the host
        :return: None
        """
        parts = record.split(",")
        if parts[1] == ip:
            self.ts = float(parts[0])
            self.ip = parts[1]
            self.cpu_percent = float(parts[2])
            self.mem_current = float(parts[3])
            self.mem_total = float(parts[4])
            self.mem_percent = float(parts[5])
            self.blk_read = float(parts[6])
            self.blk_write = float(parts[7])
            self.net_rx = float(parts[8])
            self.net_tx = float(parts[9])
            self.container_name = ""
            self.pids = int(round(float(parts[10])))
            self.timestamp = str(parts[0])

    def to_kafka_record(self, ip: str) -> str:
        """
        Converts the DTO to a kafka record

        :param ip: the ip to add to the DTO
        :return: the Kafka record
        """
        ts = time.time()
        record = f"{ts},{ip},{self.cpu_percent},{self.mem_current}," \
                 f"{self.mem_total},{self.mem_percent}," \
                 f"{self.blk_read},{self.blk_write}," \
                 f"{self.net_rx},{self.net_tx},{self.pids}"
        return record

    def copy(self) -> "DockerStats":
        """
        :return: a copy of the object
        """
        c = DockerStats(
            pids=self.pids, timestamp=self.timestamp, cpu_percent=self.cpu_percent, mem_current=self.mem_current,
            mem_total=self.mem_total, mem_percent=self.mem_percent, blk_read=self.blk_read, blk_write=self.blk_write,
            net_rx=self.net_rx, net_tx=self.net_tx, container_name=self.container_name, ip=self.ip, ts=self.ts)
        return c

    def get_deltas(self, stats_prime: "DockerStats") -> Tuple[List[int], List[str]]:
        """
        Get the deltas between two stats objects

        :param stats_prime: the stats object to compare with
        :param max_counter: the maximum counter_value
        :return: the deltas and the labels
        """
        deltas = [
            int(round(stats_prime.cpu_percent - self.cpu_percent)),
            int(int(stats_prime.mem_current - self.mem_current)),
            int(int(stats_prime.mem_total - self.mem_total)),
            int(round(stats_prime.mem_percent - self.mem_percent)),
            int(int(stats_prime.blk_read - self.blk_read)),
            int(int(stats_prime.blk_write - self.blk_write)),
            int(int(stats_prime.net_rx - self.net_rx)),
            int(int(stats_prime.net_tx - self.net_tx)),
            int(int(stats_prime.pids - self.pids))
        ]
        labels = [
            "cpu_percent", "mem_current", "mem_total", "mem_percent", "blk_read", "blk_write", "net_rx",
            "net_tx", "pids"
        ]
        return deltas, labels

    def get_values(self) -> Tuple[List[int], List[str]]:
        """
        Get the current values

        :return: the values and the labels
        """
        values = [
            int(round(self.cpu_percent)), int(self.mem_current),
            int(self.mem_total),
            int(self.mem_percent),
            int(self.blk_read), int(self.blk_write),
            int(self.net_rx),
            int(self.net_tx), int(self.pids)
        ]
        labels = [
            "cpu_percent", "mem_current", "mem_total", "mem_percent", "blk_read", "blk_write", "net_rx",
            "net_tx", "pids"
        ]
        return values, labels

    def num_attributes(self) -> int:
        """
        :return: The number of attributes of the DTO
        """
        return 13

    @staticmethod
    def schema() -> "DockerStats":
        """
        :return: get the schema of the DTO
        """
        return DockerStats()

    @staticmethod
    def from_json_file(json_file_path: str) -> "DockerStats":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return DockerStats.from_dict(json.loads(json_str))



from typing import Any
import pytest
import pytest_mock
from subprocess import CompletedProcess
from csle_collector.ossec_ids_manager.ossec_ids_manager_pb2 import OSSECIdsMonitorDTO, OSSECIdsLogDTO
from csle_collector.ossec_ids_manager.dao.ossec_ids_alert_counters import OSSECIdsAlertCounters
from csle_collector.ossec_ids_manager.ossec_ids_manager import OSSECIdsManagerServicer
import csle_collector.ossec_ids_manager.query_ossec_ids_manager


class TestOSSECIDSManagerSuite:
    """
    Test suite for the OSSEC IDS manager
    """

    @pytest.fixture(scope='module')
    def grpc_add_to_server(self) -> Any:
        """
        Necessary fixture for pytest-grpc

        :return: the add_servicer_to_server function
        """
        from csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc import add_OSSECIdsManagerServicer_to_server
        return add_OSSECIdsManagerServicer_to_server

    @pytest.fixture(scope='module')
    def grpc_servicer(self) -> OSSECIdsManagerServicer:
        """
        Necessary fixture for pytest-grpc

        :return: the ossec manager servicer
        """
        return OSSECIdsManagerServicer()

    @pytest.fixture(scope='module')
    def grpc_stub_cls(self, grpc_channel):
        """
        Necessary fixture for pytest-grpc

        :param grpc_channel: the grpc channel for testing
        :return: the stub to the service
        """
        from csle_collector.ossec_ids_manager.ossec_ids_manager_pb2_grpc import OSSECIdsManagerStub
        return OSSECIdsManagerStub

    def test_getOSSECIdsAlerts(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the getOSSECIdsAlerts grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """

        def read_ossec_ids_data(timestamp: float):
            if timestamp == 0:
                return OSSECIdsAlertCounters()
            else:
                alerts = OSSECIdsAlertCounters()
                alerts.severe_alerts = 100
                alerts.warning_alerts = 200
                alerts.total_alerts = 300
                alerts.alerts_weighted_by_level = 400
                alerts.ip = "test"
                alerts.ts = 0.0
                return alerts

        read_data_mock = mocker.MagicMock(side_effect=read_ossec_ids_data)
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSecManagerUtil.read_ossec_ids_data',
                     side_effect=read_data_mock)
        response: OSSECIdsLogDTO = csle_collector.ossec_ids_manager.query_ossec_ids_manager.get_ossec_ids_alerts(
            stub=grpc_stub, timestamp=0, log_file_path="")
        assert response.attempted_admin_alerts == 0
        assert response.total_alerts == 0
        assert response.warning_alerts == 0
        assert response.severe_alerts == 0
        assert response.alerts_weighted_by_level == 0
        assert response.level_0_alerts == 0
        assert response.level_1_alerts == 0
        assert response.level_2_alerts == 0
        assert response.level_3_alerts == 0
        assert response.level_4_alerts == 0
        assert response.level_5_alerts == 0
        assert response.level_6_alerts == 0
        assert response.level_7_alerts == 0
        assert response.level_8_alerts == 0
        assert response.level_9_alerts == 0
        assert response.level_10_alerts == 0
        assert response.level_11_alerts == 0
        assert response.level_12_alerts == 0
        assert response.level_13_alerts == 0
        assert response.level_14_alerts == 0
        assert response.level_15_alerts == 0
        assert response.invalid_login_alerts == 0
        assert response.authentication_success_alerts == 0
        assert response.authentication_failed_alerts == 0
        assert response.connection_attempt_alerts == 0
        assert response.attacks_alerts == 0
        assert response.adduser_alerts == 0
        assert response.sshd_alerts == 0
        assert response.ids_alerts == 0
        assert response.firewall_alerts == 0
        assert response.squid_alerts == 0
        assert response.apache_alerts == 0
        assert response.syslog_alerts == 0
        response = csle_collector.ossec_ids_manager.query_ossec_ids_manager.get_ossec_ids_alerts(
            stub=grpc_stub, timestamp=1, log_file_path="")
        assert response.total_alerts == 300
        assert response.warning_alerts == 200
        assert response.severe_alerts == 100
        assert response.alerts_weighted_by_level == 400

    def test_startOSSECIdsMonitor(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startOSSECIdsMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.ossec_ids_manager.threads.ossec_ids_monitor_thread.OSSecIdsMonitorThread.run',
                     return_value=True)
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_ossec_running',
                     return_value=True)
        kafka_ip = "test_kafka_ip"
        kafka_port = 9292
        log_file_path = "test_log_file_path"
        time_step_len_seconds = 30
        response: OSSECIdsMonitorDTO = csle_collector.ossec_ids_manager.query_ossec_ids_manager.start_ossec_ids_monitor(
            stub=grpc_stub, kafka_ip=kafka_ip, kafka_port=kafka_port, log_file_path=log_file_path,
            time_step_len_seconds=time_step_len_seconds)
        assert response.monitor_running
        assert response.ossec_ids_running
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_ossec_running',
                     return_value=False)
        response = csle_collector.ossec_ids_manager.query_ossec_ids_manager.start_ossec_ids_monitor(
            stub=grpc_stub, kafka_ip=kafka_ip, kafka_port=kafka_port, log_file_path=log_file_path,
            time_step_len_seconds=time_step_len_seconds)
        assert response.monitor_running
        assert not response.ossec_ids_running

    def test_stopOSSECIdsMonitor(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopOSSECIdsMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_ossec_running',
                     return_value=True)
        response: OSSECIdsMonitorDTO = csle_collector.ossec_ids_manager.query_ossec_ids_manager.stop_ossec_ids_monitor(
            stub=grpc_stub)
        assert not response.monitor_running
        assert response.ossec_ids_running
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_ossec_running',
                     return_value=False)
        response = csle_collector.ossec_ids_manager.query_ossec_ids_manager.stop_ossec_ids_monitor(
            stub=grpc_stub)
        assert not response.monitor_running
        assert not response.ossec_ids_running

    def test_stopOSSECIds(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopOSSECIdsMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('subprocess.run', return_value=CompletedProcess(args="", returncode=0, stdout="testout",
                                                                     stderr="testerr"))
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_monitor_running',
                     return_value=False)
        response: OSSECIdsMonitorDTO = csle_collector.ossec_ids_manager.query_ossec_ids_manager.stop_ossec_ids(
            stub=grpc_stub)
        assert not response.monitor_running
        assert not response.ossec_ids_running
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_monitor_running',
                     return_value=True)
        response = csle_collector.ossec_ids_manager.query_ossec_ids_manager.stop_ossec_ids(
            stub=grpc_stub)
        assert response.monitor_running
        assert not response.ossec_ids_running

    def test_startOSSECIds(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startOSSECIdsMonitor grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('subprocess.run', return_value=CompletedProcess(args="", returncode=0, stdout="testout",
                                                                     stderr="testerr"))
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_monitor_running',
                     return_value=False)
        response: OSSECIdsMonitorDTO = csle_collector.ossec_ids_manager.query_ossec_ids_manager.start_ossec_ids(
            stub=grpc_stub)
        assert not response.monitor_running
        assert response.ossec_ids_running
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_monitor_running',
                     return_value=True)
        response = csle_collector.ossec_ids_manager.query_ossec_ids_manager.start_ossec_ids(
            stub=grpc_stub)
        assert response.monitor_running
        assert response.ossec_ids_running

    def test_getOSSECIdsMonitorStatus(self, grpc_stub, mocker) -> None:
        """
        Tests the getOSSECIdsMonitorStatus grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_ossec_running',
                     return_value=False)
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_monitor_running',
                     return_value=False)
        response: OSSECIdsMonitorDTO = \
            csle_collector.ossec_ids_manager.query_ossec_ids_manager.get_ossec_ids_monitor_status(stub=grpc_stub)
        assert not response.monitor_running
        assert not response.ossec_ids_running

        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_ossec_running',
                     return_value=True)
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_monitor_running',
                     return_value=False)
        response = \
            csle_collector.ossec_ids_manager.query_ossec_ids_manager.get_ossec_ids_monitor_status(stub=grpc_stub)
        assert not response.monitor_running
        assert response.ossec_ids_running

        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_ossec_running',
                     return_value=False)
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_monitor_running',
                     return_value=True)
        response = \
            csle_collector.ossec_ids_manager.query_ossec_ids_manager.get_ossec_ids_monitor_status(stub=grpc_stub)
        assert response.monitor_running
        assert not response.ossec_ids_running

        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_ossec_running',
                     return_value=True)
        mocker.patch('csle_collector.ossec_ids_manager.ossec_ids_manager.OSSECIdsManagerServicer._is_monitor_running',
                     return_value=True)
        response = \
            csle_collector.ossec_ids_manager.query_ossec_ids_manager.get_ossec_ids_monitor_status(stub=grpc_stub)
        assert response.monitor_running
        assert response.ossec_ids_running



from csle_collector.ossec_ids_manager.dao.ossec_ids_alert import OSSECIDSAlert
from csle_collector.ossec_ids_manager.dao.ossec_ids_alert_counters import OSSECIdsAlertCounters


class TestOSSECIdsManagerDaoSuite:
    """
    Test suite for datasets data access objects (DAOs) in ossec_ids_manager
    """

    def test_ossec_ids_alert(self, example_ossec_ids_alert: OSSECIDSAlert) -> None:
        """
        Tests creation and dict conversion of the OSSECIDSAlert DAO

        :param example_ossec_ids_alert: an example OSSECIDSAlert
        :return: None
        """
        assert isinstance(example_ossec_ids_alert.to_dict(), dict)
        assert isinstance(OSSECIDSAlert.from_dict(example_ossec_ids_alert.to_dict()),
                          OSSECIDSAlert)
        assert (OSSECIDSAlert.from_dict(example_ossec_ids_alert.to_dict()).to_dict()
                == example_ossec_ids_alert.to_dict())
        assert (OSSECIDSAlert.from_dict(example_ossec_ids_alert.to_dict())
                == example_ossec_ids_alert)

    def test_ossec_ids_alert_counters(self, example_ossec_ids_alert_counters: OSSECIdsAlertCounters) -> None:
        """
        Tests creation and dict conversion of the OSSECIdsAlertCounters DAO

        :param example_ossec_ids_alert_counters: an example OSSECIdsAlertCounters
        :return: None
        """
        assert isinstance(example_ossec_ids_alert_counters.to_dict(), dict)
        assert isinstance(OSSECIdsAlertCounters.from_dict(example_ossec_ids_alert_counters.to_dict()),
                          OSSECIdsAlertCounters)
        assert (OSSECIdsAlertCounters.from_dict(example_ossec_ids_alert_counters.to_dict()).to_dict()
                == example_ossec_ids_alert_counters.to_dict())
        assert (OSSECIdsAlertCounters.from_dict(example_ossec_ids_alert_counters.to_dict())
                == example_ossec_ids_alert_counters)



from typing import Dict, Any, List
from csle_collector.client_manager.dao.arrival_config import ArrivalConfig
from csle_collector.client_manager.dao.client_arrival_type import ClientArrivalType
import csle_collector.client_manager.client_manager_pb2


class EPTMPArrivalConfig(ArrivalConfig):
    """
    DTO representing the configuration of a homogenous poisson arrival process with an
    Exponential-Polynomial-Trigonometric rate function having Multiple Periodicities
    """

    def __init__(self, thetas: List[float], gammas: List[float], phis: List[float], omegas: List[float]):
        """
        Initializes the object

        :param thetas: represent the overall trend in frequency of events over a long time frame
        :param gammas: amplitudes
        :param phis: period shifts
        :param omegas: frequencies
        """
        self.thetas = thetas
        self.gammas = gammas
        self.phis = phis
        self.omegas = omegas
        super(EPTMPArrivalConfig, self).__init__(client_arrival_type=ClientArrivalType.EPTMP)

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"Arrival type: Poisson process with EPTMP rate function, " \
               f"thetas: {self.thetas}, gammas: {self.gammas}, phis: {self.phis}, omegas: {self.omegas}, " \
               f"client_arrival_type: {self.client_arrival_type}"

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Any] = {}
        d["thetas"] = self.thetas
        d["gammas"] = self.gammas
        d["phis"] = self.phis
        d["omegas"] = self.omegas
        d["client_arrival_type"] = self.client_arrival_type
        return d

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "EPTMPArrivalConfig":
        """
        Converts a dict representation of the object to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = EPTMPArrivalConfig(thetas=d["thetas"], gammas=d["gammas"], phis=d["phis"], omegas=d["omegas"])
        return obj

    def to_grpc_object(self) -> csle_collector.client_manager.client_manager_pb2.EPTMPArrivalConfigDTO:
        """
        :return: a GRPC serializable version of the object
        """
        return csle_collector.client_manager.client_manager_pb2.EPTMPArrivalConfigDTO(
            thetas=self.thetas, gammas=self.gammas, phis=self.phis, omegas=self.omegas
        )

    @staticmethod
    def from_grpc_object(obj: csle_collector.client_manager.client_manager_pb2.EPTMPArrivalConfigDTO) \
            -> "EPTMPArrivalConfig":
        """
        Instantiates the object from a GRPC DTO

        :param obj: the object to instantiate from
        :return: the instantiated object
        """
        return EPTMPArrivalConfig(thetas=list(obj.thetas), gammas=list(obj.gammas), phis=list(obj.phis),
                                  omegas=list(obj.omegas))

    @staticmethod
    def from_json_file(json_file_path: str) -> "EPTMPArrivalConfig":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return EPTMPArrivalConfig.from_dict(json.loads(json_str))



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class StopRyuMsg(google.protobuf.message.Message):
    """Message that the client sends to stop Ryu"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopRyuMsg = StopRyuMsg

@typing_extensions.final
class StartRyuMsg(google.protobuf.message.Message):
    """Message that the client sends to start Ryu"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PORT_FIELD_NUMBER: builtins.int
    WEB_PORT_FIELD_NUMBER: builtins.int
    CONTROLLER_FIELD_NUMBER: builtins.int
    port: builtins.int
    web_port: builtins.int
    controller: builtins.str
    def __init__(
        self,
        *,
        port: builtins.int = ...,
        web_port: builtins.int = ...,
        controller: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["controller", b"controller", "port", b"port", "web_port", b"web_port"]) -> None: ...

global___StartRyuMsg = StartRyuMsg

@typing_extensions.final
class StopRyuMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the Ryu monitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopRyuMonitorMsg = StopRyuMonitorMsg

@typing_extensions.final
class StartRyuMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to start the Ryu monitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_FIELD_NUMBER: builtins.int
    kafka_ip: builtins.str
    kafka_port: builtins.int
    time_step_len: builtins.int
    def __init__(
        self,
        *,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        time_step_len: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "time_step_len", b"time_step_len"]) -> None: ...

global___StartRyuMonitorMsg = StartRyuMonitorMsg

@typing_extensions.final
class GetRyuStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the Ryu status"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetRyuStatusMsg = GetRyuStatusMsg

@typing_extensions.final
class RyuDTO(google.protobuf.message.Message):
    """Message that the server returns when requested by the client, contains info about Ryu"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RYU_RUNNING_FIELD_NUMBER: builtins.int
    MONITOR_RUNNING_FIELD_NUMBER: builtins.int
    PORT_FIELD_NUMBER: builtins.int
    WEB_PORT_FIELD_NUMBER: builtins.int
    CONTROLLER_FIELD_NUMBER: builtins.int
    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_FIELD_NUMBER: builtins.int
    ryu_running: builtins.bool
    monitor_running: builtins.bool
    port: builtins.int
    web_port: builtins.int
    controller: builtins.str
    kafka_ip: builtins.str
    kafka_port: builtins.int
    time_step_len: builtins.int
    def __init__(
        self,
        *,
        ryu_running: builtins.bool = ...,
        monitor_running: builtins.bool = ...,
        port: builtins.int = ...,
        web_port: builtins.int = ...,
        controller: builtins.str = ...,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        time_step_len: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["controller", b"controller", "kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "monitor_running", b"monitor_running", "port", b"port", "ryu_running", b"ryu_running", "time_step_len", b"time_step_len", "web_port", b"web_port"]) -> None: ...

global___RyuDTO = RyuDTO



from typing import List, Dict, Any
from datetime import datetime
import subprocess
from csle_collector.ossec_ids_manager.dao.ossec_ids_alert import OSSECIDSAlert
from csle_collector.ossec_ids_manager.dao.ossec_ids_alert_counters import OSSECIdsAlertCounters
import csle_collector.ossec_ids_manager.ossec_ids_manager_pb2
import csle_collector.constants.constants as constants


class OSSecManagerUtil:
    """
    Class with utility functions related to the OSSEC IDS Manager
    """

    @staticmethod
    def check_ossec_ids_alerts() -> List[OSSECIDSAlert]:
        """
        Reads alerts from the OSSEC IDS alerts log

        :return: a list of alerts
        """
        cmd = constants.OSSEC.TAIL_ALERTS_COMMAND + " " + constants.OSSEC.OSSEC_ALERTS_FILE
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        alerts: List[OSSECIDSAlert] = []
        timestamp = datetime.now()
        groups: List[str] = []
        host = " "
        ip = " "
        ruleid = " "
        level = 1
        desc = " "
        src = " "
        user = " "
        linesmatched = 0
        for line in result.stdout.split("\n"):
            linematched = 0

            # Test for matches. A line will have more than one matching RE.
            if constants.OSSEC.ALERTLINE_REGEX.match(line):
                linematched = 1
                match = constants.OSSEC.ALERTLINE_REGEX.match(line)
                if match is not None:
                    groupstr = match.group(2).rstrip(',')
                    groups = groupstr.split(',')

            if constants.OSSEC.DATELINEREGEX.match(line):
                linematched = 1
                match = constants.OSSEC.DATELINEREGEX.match(line)
                if match is not None:
                    datestr = match.group(0)
                    timestamp = datetime.strptime(datestr, "%Y %b %d %H:%M:%S")

            if constants.OSSEC.HOSTLINE_REGEX.match(line):
                linematched += 1
                match = constants.OSSEC.HOSTLINE_REGEX.match(line)
                if match is not None:
                    host = match.group(1)
                    ip = match.group(2)

            if constants.OSSEC.SERVHOSTLINE_REGEX.match(line):
                linematched += 1
                match = constants.OSSEC.SERVHOSTLINE_REGEX.match(line)
                if match is not None:
                    host = match.group(1)
                    ip = '0.0.0.0'

            if constants.OSSEC.RULELINE_REGEX.match(line):
                linematched += 1
                match = constants.OSSEC.RULELINE_REGEX.match(line)
                if match is not None:
                    ruleid = match.group(1)
                    level = int(match.group(2))
                    desc = match.group(3)

            if constants.OSSEC.SRCIPLINE_REGEX.match(line):
                linematched += 1
                match = constants.OSSEC.SRCIPLINE_REGEX.match(line)
                if match is not None:
                    src = match.group(1)

            if constants.OSSEC.USERLINE_REGEX.match(line):
                linematched += 1
                match = constants.OSSEC.USERLINE_REGEX.match(line)
                if match is not None:
                    user = match.group(1)

            linesmatched += linematched

            if linematched == 0 and linesmatched > 0:
                if len(line) <= 1:
                    alert = OSSECIDSAlert(timestamp=datetime.timestamp(timestamp), groups=groups, host=host, ip=ip,
                                          rule_id=ruleid, level=level, descr=desc, src=src, user=user)
                    print(alert)
                    alerts.append(alert)
                    linesmatched = 0
                    timestamp = datetime.now()
                    groups = []
                    host = " "
                    ip = " "
                    ruleid = " "
                    level = 1
                    desc = " "
                    src = " "
                    user = " "
        return alerts

    @staticmethod
    def read_ossec_ids_data(episode_last_alert_ts: float) -> OSSECIdsAlertCounters:
        """
        Measures metrics from the OSSEC ids

        :param env_config: environment configuration
        :param episode_last_alert_ts: timestamp when the episode started
        :return: ids statistics
        """

        # Read OSSEC IDS data
        alerts = OSSecManagerUtil.check_ossec_ids_alerts()

        # Filter IDS data from beginning of episode
        alerts = list(filter(lambda x: x.timestamp > episode_last_alert_ts, alerts))

        counters = OSSECIdsAlertCounters()
        counters.count(alerts)

        return counters

    @staticmethod
    def ossec_ids_monitor_dto_to_dict(
            ossec_ids_monitor_dto: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO) \
            -> Dict[str, Any]:
        """
        Converts a OSSECIdsMonitorDTO to a dict

        :param ossec_ids_monitor_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d = {}
        d["monitor_running"] = ossec_ids_monitor_dto.monitor_running
        d["ossec_ids_running"] = ossec_ids_monitor_dto.ossec_ids_running
        return d

    @staticmethod
    def ossec_ids_log_dto_to_dict(
            ossec_ids_log_dto: csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO) \
            -> Dict[str, Any]:
        """
        Converts a OSSECIdsLogDTO to a dict

        :param ossec_ids_log_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d: Dict[str, Any] = {}
        d["timestamp"] = ossec_ids_log_dto.timestamp
        d["ip"] = ossec_ids_log_dto.ip
        d["attempted_admin_alerts"] = ossec_ids_log_dto.attempted_admin_alerts
        d["total_alerts"] = ossec_ids_log_dto.total_alerts
        d["warning_alerts"] = ossec_ids_log_dto.warning_alerts
        d["severe_alerts"] = ossec_ids_log_dto.severe_alerts
        d["alerts_weighted_by_level"] = ossec_ids_log_dto.alerts_weighted_by_level
        d["level_0_alerts"] = ossec_ids_log_dto.level_0_alerts
        d["level_1_alerts"] = ossec_ids_log_dto.level_1_alerts
        d["level_2_alerts"] = ossec_ids_log_dto.level_2_alerts
        d["level_3_alerts"] = ossec_ids_log_dto.level_3_alerts
        d["level_4_alerts"] = ossec_ids_log_dto.level_4_alerts
        d["level_5_alerts"] = ossec_ids_log_dto.level_5_alerts
        d["level_6_alerts"] = ossec_ids_log_dto.level_6_alerts
        d["level_7_alerts"] = ossec_ids_log_dto.level_7_alerts
        d["level_8_alerts"] = ossec_ids_log_dto.level_8_alerts
        d["level_9_alerts"] = ossec_ids_log_dto.level_9_alerts
        d["level_10_alerts"] = ossec_ids_log_dto.level_10_alerts
        d["level_11_alerts"] = ossec_ids_log_dto.level_11_alerts
        d["level_12_alerts"] = ossec_ids_log_dto.level_12_alerts
        d["level_13_alerts"] = ossec_ids_log_dto.level_13_alerts
        d["level_14_alerts"] = ossec_ids_log_dto.level_14_alerts
        d["level_15_alerts"] = ossec_ids_log_dto.level_15_alerts
        d["invalid_login_alerts"] = ossec_ids_log_dto.invalid_login_alerts
        d["authentication_success_alerts"] = ossec_ids_log_dto.authentication_success_alerts
        d["authentication_failed_alerts"] = ossec_ids_log_dto.authentication_failed_alerts
        d["connection_attempt_alerts"] = ossec_ids_log_dto.connection_attempt_alerts
        d["attacks_alerts"] = ossec_ids_log_dto.attacks_alerts
        d["adduser_alerts"] = ossec_ids_log_dto.adduser_alerts
        d["sshd_alerts"] = ossec_ids_log_dto.sshd_alerts
        d["ids_alerts"] = ossec_ids_log_dto.ids_alerts
        d["firewall_alerts"] = ossec_ids_log_dto.firewall_alerts
        d["squid_alerts"] = ossec_ids_log_dto.squid_alerts
        d["apache_alerts"] = ossec_ids_log_dto.apache_alerts
        d["syslog_alerts"] = ossec_ids_log_dto.syslog_alerts
        return d

    @staticmethod
    def ossec_ids_monitor_dto_from_dict(d: Dict[str, Any]) \
            -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
        """
        Converts a dict representation of a OSSECIDSMonitorDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        ossec_ids_monitor_dto = csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO()
        ossec_ids_monitor_dto.monitor_running = d["monitor_running"]
        ossec_ids_monitor_dto.ossec_ids_running = d["ossec_ids_running"]
        return ossec_ids_monitor_dto

    @staticmethod
    def ossec_ids_log_dto_from_dict(d: Dict[str, Any]) \
            -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO:
        """
        Converts a dict representation of a OSSECIDSLogDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        ossec_ids_log_dto = csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO()
        ossec_ids_log_dto.timestamp = d["timestamp"]
        ossec_ids_log_dto.ip = d["ip"]
        ossec_ids_log_dto.attempted_admin_alerts = d["attempted_admin_alerts"]
        ossec_ids_log_dto.total_alerts = d["total_alerts"]
        ossec_ids_log_dto.warning_alerts = d["warning_alerts"]
        ossec_ids_log_dto.severe_alerts = d["severe_alerts"]
        ossec_ids_log_dto.alerts_weighted_by_level = d["alerts_weighted_by_level"]
        ossec_ids_log_dto.level_0_alerts = d["level_0_alerts"]
        ossec_ids_log_dto.level_1_alerts = d["level_1_alerts"]
        ossec_ids_log_dto.level_2_alerts = d["level_2_alerts"]
        ossec_ids_log_dto.level_3_alerts = d["level_3_alerts"]
        ossec_ids_log_dto.level_4_alerts = d["level_4_alerts"]
        ossec_ids_log_dto.level_5_alerts = d["level_5_alerts"]
        ossec_ids_log_dto.level_6_alerts = d["level_6_alerts"]
        ossec_ids_log_dto.level_7_alerts = d["level_7_alerts"]
        ossec_ids_log_dto.level_8_alerts = d["level_8_alerts"]
        ossec_ids_log_dto.level_9_alerts = d["level_9_alerts"]
        ossec_ids_log_dto.level_10_alerts = d["level_10_alerts"]
        ossec_ids_log_dto.level_11_alerts = d["level_11_alerts"]
        ossec_ids_log_dto.level_12_alerts = d["level_12_alerts"]
        ossec_ids_log_dto.level_13_alerts = d["level_13_alerts"]
        ossec_ids_log_dto.level_14_alerts = d["level_14_alerts"]
        ossec_ids_log_dto.level_15_alerts = d["level_15_alerts"]
        ossec_ids_log_dto.invalid_login_alerts = d["invalid_login_alerts"]
        ossec_ids_log_dto.authentication_success_alerts = d["authentication_success_alerts"]
        ossec_ids_log_dto.authentication_failed_alerts = d["authentication_failed_alerts"]
        ossec_ids_log_dto.connection_attempt_alerts = d["connection_attempt_alerts"]
        ossec_ids_log_dto.attacks_alerts = d["attacks_alerts"]
        ossec_ids_log_dto.adduser_alerts = d["adduser_alerts"]
        ossec_ids_log_dto.sshd_alerts = d["sshd_alerts"]
        ossec_ids_log_dto.ids_alerts = d["ids_alerts"]
        ossec_ids_log_dto.firewall_alerts = d["firewall_alerts"]
        ossec_ids_log_dto.squid_alerts = d["squid_alerts"]
        ossec_ids_log_dto.apache_alerts = d["apache_alerts"]
        ossec_ids_log_dto.syslog_alerts = d["syslog_alerts"]
        return ossec_ids_log_dto

    @staticmethod
    def ossec_ids_log_dto_empty() \
            -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO:
        """
        :return: an empty OSSECIdsLogDTO
        """
        ossec_ids_log_dto = csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsLogDTO()
        ossec_ids_log_dto.timestamp = 0.0
        ossec_ids_log_dto.ip = ""
        ossec_ids_log_dto.attempted_admin_alerts = 0
        ossec_ids_log_dto.total_alerts = 0
        ossec_ids_log_dto.warning_alerts = 0
        ossec_ids_log_dto.severe_alerts = 0
        ossec_ids_log_dto.alerts_weighted_by_level = 0
        ossec_ids_log_dto.level_0_alerts = 0
        ossec_ids_log_dto.level_1_alerts = 0
        ossec_ids_log_dto.level_2_alerts = 0
        ossec_ids_log_dto.level_3_alerts = 0
        ossec_ids_log_dto.level_4_alerts = 0
        ossec_ids_log_dto.level_5_alerts = 0
        ossec_ids_log_dto.level_6_alerts = 0
        ossec_ids_log_dto.level_7_alerts = 0
        ossec_ids_log_dto.level_8_alerts = 0
        ossec_ids_log_dto.level_9_alerts = 0
        ossec_ids_log_dto.level_10_alerts = 0
        ossec_ids_log_dto.level_11_alerts = 0
        ossec_ids_log_dto.level_12_alerts = 0
        ossec_ids_log_dto.level_13_alerts = 0
        ossec_ids_log_dto.level_14_alerts = 0
        ossec_ids_log_dto.level_15_alerts = 0
        ossec_ids_log_dto.invalid_login_alerts = 0
        ossec_ids_log_dto.authentication_success_alerts = 0
        ossec_ids_log_dto.authentication_failed_alerts = 0
        ossec_ids_log_dto.connection_attempt_alerts = 0
        ossec_ids_log_dto.attacks_alerts = 0
        ossec_ids_log_dto.adduser_alerts = 0
        ossec_ids_log_dto.sshd_alerts = 0
        ossec_ids_log_dto.ids_alerts = 0
        ossec_ids_log_dto.firewall_alerts = 0
        ossec_ids_log_dto.squid_alerts = 0
        ossec_ids_log_dto.apache_alerts = 0
        ossec_ids_log_dto.syslog_alerts = 0
        return ossec_ids_log_dto

    @staticmethod
    def ossec_ids_monitor_dto_empty() -> csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO:
        """
        :return: An empty OSSECIdsMonitorDTO
        """
        ossec_ids_monitor_dto = csle_collector.ossec_ids_manager.ossec_ids_manager_pb2.OSSECIdsMonitorDTO()
        ossec_ids_monitor_dto.monitor_running = False
        ossec_ids_monitor_dto.ossec_ids_running = False
        return ossec_ids_monitor_dto



from typing import List
import logging
import os
import subprocess
from concurrent import futures
import grpc
import socket
import netifaces
import io
import csle_collector.traffic_manager.traffic_manager_pb2_grpc
import csle_collector.traffic_manager.traffic_manager_pb2
import csle_collector.constants.constants as constants


class TrafficManagerServicer(csle_collector.traffic_manager.traffic_manager_pb2_grpc.TrafficManagerServicer):
    """
    gRPC server for managing a traffic generator. Allows to start/stop the script and also to query the
    state of the script.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        logging.basicConfig(filename=f"{constants.LOG_FILES.TRAFFIC_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.TRAFFIC_MANAGER_LOG_FILE}", level=logging.INFO)
        self.hostname = socket.gethostname()
        try:
            self.ip = netifaces.ifaddresses(constants.INTERFACES.ETH0)[netifaces.AF_INET][0][constants.INTERFACES.ADDR]
        except Exception:
            self.ip = socket.gethostbyname(self.hostname)
        logging.info(f"Setting up TrafficManager hostname: {self.hostname} ip: {self.ip}")

    @staticmethod
    def _get_traffic_status() -> bool:
        """
        Utility method to get the status of the traffic generator script

        :return: status of the traffic generator
        """
        cmd = constants.TRAFFIC_GENERATOR.CHECK_IF_TRAFFIC_GENERATOR_IS_RUNNING
        output = subprocess.run(cmd.split(" "), capture_output=True, text=True).stdout
        running = constants.TRAFFIC_GENERATOR.TRAFFIC_GENERATOR_FILE_NAME in output
        return running

    @staticmethod
    def _read_traffic_script() -> str:
        """
        :return: the traffic generator script file
        """
        try:
            with io.open(f"/{constants.TRAFFIC_GENERATOR.TRAFFIC_GENERATOR_FILE_NAME}", 'r', encoding='utf-8') as f:
                script_file_str = f.read()
                return script_file_str
        except Exception as e:
            logging.info(f"Could not read the script file: {str(e)}, {repr(e)}")
            return ""

    @staticmethod
    def _create_traffic_script(commands: List[str], sleep_time: int) -> None:
        """
        Utility method to create the traffic script based on the list of commands

        :param commands: the list of commands
        :param sleep_time: the sleep time
        :return: None
        """
        # File contents
        script_file = ""
        script_file = script_file + "#!/bin/bash\n"
        script_file = script_file + "while [ 1 ]\n"
        script_file = script_file + "do\n"
        script_file = script_file + "    sleep {}\n".format(sleep_time)
        for cmd in commands:
            script_file = script_file + "    " + cmd + "\n"
            script_file = script_file + "    sleep {}\n".format(sleep_time)
        script_file = script_file + "done\n"

        # Remove old file if exists
        cmd = constants.TRAFFIC_GENERATOR.REMOVE_OLD_TRAFFIC_GENERATOR_FILE
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        logging.info(f"Removed old file, stdout: {result.stdout}, stderr: {result.stderr}")

        # Create file
        cmd = constants.TRAFFIC_GENERATOR.CREATE_TRAFFIC_GENERATOR_FILE
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        logging.info(f"Created new file, stdout: {result.stdout}, stderr: {result.stderr}")

        # Make executable
        cmd = constants.TRAFFIC_GENERATOR.MAKE_TRAFFIC_GENERATOR_FILE_EXECUTABLE
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        logging.info(f"Changed permissions, stdout: {result.stdout}, stderr: {result.stderr}")

        # Write traffic generation script file
        with io.open(f"/{constants.TRAFFIC_GENERATOR.TRAFFIC_GENERATOR_FILE_NAME}", 'w', encoding='utf-8') as f:
            f.write(script_file)

    def getTrafficStatus(self, request: csle_collector.traffic_manager.traffic_manager_pb2.GetTrafficStatusMsg,
                         context: grpc.ServicerContext) \
            -> csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO:
        """
        Gets the state of the traffic manager

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a TrafficDTO with the state of the traffic manager
        """
        logging.info("Getting traffic manager status")
        running = TrafficManagerServicer._get_traffic_status()
        script_file_str = TrafficManagerServicer._read_traffic_script()
        logging.info("Returning traffic manager status")
        traffic_dto = csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO(running=running,
                                                                                    script=script_file_str)
        return traffic_dto

    def stopTraffic(self, request: csle_collector.traffic_manager.traffic_manager_pb2.StopTrafficMsg,
                    context: grpc.ServicerContext):
        """
        Stops the traffic generator

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a traffic DTO with the state of the traffic generator
        """
        logging.info("Stopping the traffic generator script")
        cmd = "sudo pkill -f traffic_generator.sh"
        os.system(cmd)
        script_file_str = TrafficManagerServicer._read_traffic_script()
        logging.info("Traffic generator stopped")
        return csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO(running=False, script=script_file_str)

    def startTraffic(self, request: csle_collector.traffic_manager.traffic_manager_pb2.StartTrafficMsg,
                     context: grpc.ServicerContext) -> csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO:
        """
        Starts the traffic generator

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a TrafficDTO with the state of the traffic generator
        """
        logging.info(f"Starting the traffic generator, \n sleep_time: {request.sleepTime}, "
                     f"commands:{request.commands}")
        commands = list(request.commands)
        sleep_time = request.sleepTime
        TrafficManagerServicer._create_traffic_script(commands=commands, sleep_time=sleep_time)
        cmd = constants.TRAFFIC_GENERATOR.START_TRAFFIC_GENERATOR_CMD
        os.system(cmd)
        script_file_str = TrafficManagerServicer._read_traffic_script()
        traffic_dto = csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO(
            running=True, script=script_file_str)
        logging.info("Traffic generator started")
        return traffic_dto


def serve(port: int = 50043, log_dir: str = "/", max_workers: int = 10,
          log_file_name: str = "traffic_manager.log") -> None:
    """
    Starts the gRPC server for managing traffic scripts

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.TRAFFIC_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.TRAFFIC_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.traffic_manager.traffic_manager_pb2_grpc.add_TrafficManagerServicer_to_server(
        TrafficManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"TrafficManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



from typing import Dict, Any
import csle_collector.client_manager.client_manager_pb2


class ClientManagerUtil:
    """
    Class with utility functions related to the Client Manager
    """

    @staticmethod
    def client_dto_to_dict(clients_dto: csle_collector.client_manager.client_manager_pb2.ClientsDTO) \
            -> Dict[str, Any]:
        """
        Converts a ClientsDTO to a dict

        :param clients_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d = {}
        d["num_clients"] = clients_dto.num_clients
        d["client_process_active"] = clients_dto.client_process_active
        d["producer_active"] = clients_dto.producer_active
        d["clients_time_step_len_seconds"] = clients_dto.clients_time_step_len_seconds
        d["producer_time_step_len_seconds"] = clients_dto.producer_time_step_len_seconds
        return d

    @staticmethod
    def clients_dto_from_dict(d: Dict[str, Any]) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Converts a dict representation of a ClientsDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO()
        clients_dto.num_clients = d["num_clients"]
        clients_dto.client_process_active = d["client_process_active"]
        clients_dto.producer_active = d["producer_active"]
        clients_dto.clients_time_step_len_seconds = d["clients_time_step_len_seconds"]
        clients_dto.producer_time_step_len_seconds = d["producer_time_step_len_seconds"]
        return clients_dto

    @staticmethod
    def clients_dto_empty() -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        :return: an empty ClientsDTO
        """
        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO()
        clients_dto.num_clients = 0
        clients_dto.client_process_active = False
        clients_dto.producer_active = False
        clients_dto.clients_time_step_len_seconds = 0
        clients_dto.producer_time_step_len_seconds = 0
        return clients_dto



import datetime
from csle_base.json_serializable import JSONSerializable
from typing import Dict, Any


class FailedLoginAttempt(JSONSerializable):
    """
    Class representing a failed login event on some server in the emulation
    """

    def __init__(self):
        """
        Initializes the object
        """
        self.timestamp = None

    @staticmethod
    def parse_from_str(login_attempt_str: str) -> "FailedLoginAttempt":
        """
        Parses a failed login event DTO from a string

        :param login_attempt_str: the string to parse
        :return: the parsed DTO
        """
        failed_login_attempt_dto = FailedLoginAttempt()
        failed_login_attempt_dto.timestamp = \
            datetime.datetime.strptime(login_attempt_str, '%Y %b %d %H:%M:%S').timestamp()
        return failed_login_attempt_dto

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return "timestamp:{}".format(self.timestamp)

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Any] = {}
        d["timestamp"] = self.timestamp
        return d

    @staticmethod
    def from_dict(parsed_stats_dict: Dict[str, Any]) -> "FailedLoginAttempt":
        """
        Parses a FailedLoginAttempt object from a dict

        :param parsed_stats_dict: the dict to parse
        :return: the parsed FailedLoginAttempt object
        """
        failed_login_attempt_dto = FailedLoginAttempt()
        failed_login_attempt_dto.timestamp = parsed_stats_dict["timestamp"]
        return failed_login_attempt_dto

    @staticmethod
    def from_json_file(json_file_path: str) -> "FailedLoginAttempt":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return FailedLoginAttempt.from_dict(json.loads(json_str))



from csle_collector.client_manager.dao.constant_arrival_config import ConstantArrivalConfig
from csle_collector.client_manager.dao.sine_arrival_config import SineArrivalConfig
from csle_collector.client_manager.dao.client import Client
from csle_collector.client_manager.dao.eptmp_arrival_config import EPTMPArrivalConfig
from csle_collector.client_manager.dao.piece_wise_constant_arrival_config import PieceWiseConstantArrivalConfig
from csle_collector.client_manager.dao.spiking_arrival_config import SpikingArrivalConfig
from csle_collector.client_manager.dao.workflow_markov_chain import WorkflowMarkovChain
from csle_collector.client_manager.dao.workflow_service import WorkflowService
from csle_collector.client_manager.dao.workflows_config import WorkflowsConfig


class TestClientManagerDaoSuite:
    """
    Test suite for datasets data access objects (DAOs) in client_manager
    """

    def test_constant_arrival_config(self, example_constant_arrival_config: ConstantArrivalConfig) -> None:
        """
        Tests creation and dict conversion of the ConstantArrivalConfig DAO

        :param example_constant_arrival_config: an example ConstantArrivalConfig
        :return: None
        """
        assert isinstance(example_constant_arrival_config.to_dict(), dict)
        assert isinstance(ConstantArrivalConfig.from_dict(example_constant_arrival_config.to_dict()),
                          ConstantArrivalConfig)
        assert (ConstantArrivalConfig.from_dict(example_constant_arrival_config.to_dict()).to_dict()
                == example_constant_arrival_config.to_dict())
        assert (ConstantArrivalConfig.from_dict(example_constant_arrival_config.to_dict())
                == example_constant_arrival_config)

    def test_since_arrival_config(self, example_sine_arrival_config: SineArrivalConfig) -> None:
        """
        Tests creation and dict conversion of the SineArrivalConfig DAO

        :param example_constant_arrival_config: an example SineArrivalConfig
        :return: None
        """
        assert isinstance(example_sine_arrival_config.to_dict(), dict)
        assert isinstance(SineArrivalConfig.from_dict(example_sine_arrival_config.to_dict()),
                          SineArrivalConfig)
        assert (SineArrivalConfig.from_dict(example_sine_arrival_config.to_dict()).to_dict()
                == example_sine_arrival_config.to_dict())
        assert (SineArrivalConfig.from_dict(example_sine_arrival_config.to_dict())
                == example_sine_arrival_config)

    def test_client(self, example_client: Client) -> None:
        """
        Tests creation and dict conversion of the Client DAO

        :param example_client: an example Client
        :return: None
        """
        assert isinstance(example_client.to_dict(), dict)
        assert isinstance(Client.from_dict(example_client.to_dict()), Client)
        assert (Client.from_dict(example_client.to_dict()).to_dict() == example_client.to_dict())
        assert (Client.from_dict(example_client.to_dict()) == example_client)

    def test_eptmp(self, example_eptmp_arrival_config: EPTMPArrivalConfig) -> None:
        """
        Tests creation and dict conversion of the EPTMPArrivalConfig DAO

        :param example_eptmp_arrival_config: an example EPTMPArrivalConfig
        :return: None
        """
        assert isinstance(example_eptmp_arrival_config.to_dict(), dict)
        assert isinstance(EPTMPArrivalConfig.from_dict(example_eptmp_arrival_config.to_dict()), EPTMPArrivalConfig)
        assert (EPTMPArrivalConfig.from_dict(example_eptmp_arrival_config.to_dict()).to_dict() ==
                example_eptmp_arrival_config.to_dict())
        assert (EPTMPArrivalConfig.from_dict(example_eptmp_arrival_config.to_dict()) == example_eptmp_arrival_config)

    def test_piece_wise_constant_arrival_config(
            self, example_piece_wise_constant_arrival_config: PieceWiseConstantArrivalConfig) -> None:
        """
        Tests creation and dict conversion of the PieceWiseConstantArrivalConfig DAO

        :param example_piece_wise_constant_arrival_config: an example PieceWiseConstantArrivalConfig
        :return: None
        """
        assert isinstance(example_piece_wise_constant_arrival_config.to_dict(), dict)
        assert isinstance(PieceWiseConstantArrivalConfig.from_dict(
            example_piece_wise_constant_arrival_config.to_dict()), PieceWiseConstantArrivalConfig)
        assert (PieceWiseConstantArrivalConfig.from_dict(
            example_piece_wise_constant_arrival_config.to_dict()).to_dict()
            == example_piece_wise_constant_arrival_config.to_dict())
        assert (PieceWiseConstantArrivalConfig.from_dict(example_piece_wise_constant_arrival_config.to_dict())
                == example_piece_wise_constant_arrival_config)

    def test_example_spiking_arrival_config(
            self, example_spiking_arrival_config: SpikingArrivalConfig) -> None:
        """
        Tests creation and dict conversion of the SpikingArrivalConfig DAO

        :param example_spiking_arrival_config: an example SpikingArrivalConfig
        :return: None
        """
        assert isinstance(example_spiking_arrival_config.to_dict(), dict)
        assert isinstance(SpikingArrivalConfig.from_dict(example_spiking_arrival_config.to_dict()),
                          SpikingArrivalConfig)
        assert (SpikingArrivalConfig.from_dict(example_spiking_arrival_config.to_dict()).to_dict()
                == example_spiking_arrival_config.to_dict())
        assert (SpikingArrivalConfig.from_dict(example_spiking_arrival_config.to_dict())
                == example_spiking_arrival_config)

    def test_example_workflow_markov_chain(
            self, example_workflow_markov_chain: WorkflowMarkovChain) -> None:
        """
        Tests creation and dict conversion of the WorkflowMarkovChain DAO

        :param example_workflow_markov_chain: an example WorkflowMarkovChain
        :return: None
        """
        assert isinstance(example_workflow_markov_chain.to_dict(), dict)
        assert isinstance(WorkflowMarkovChain.from_dict(example_workflow_markov_chain.to_dict()),
                          WorkflowMarkovChain)
        assert (WorkflowMarkovChain.from_dict(example_workflow_markov_chain.to_dict()).to_dict()
                == example_workflow_markov_chain.to_dict())
        assert (WorkflowMarkovChain.from_dict(example_workflow_markov_chain.to_dict())
                == example_workflow_markov_chain)

    def test_example_workflow_service(self, example_workflow_service: WorkflowService) -> None:
        """
        Tests creation and dict conversion of the WorkflowService DAO

        :param example_workflow_service: an example WorkflowService
        :return: None
        """
        assert isinstance(example_workflow_service.to_dict(), dict)
        assert isinstance(WorkflowService.from_dict(example_workflow_service.to_dict()), WorkflowService)
        assert (WorkflowService.from_dict(example_workflow_service.to_dict()).to_dict()
                == example_workflow_service.to_dict())
        assert (WorkflowService.from_dict(example_workflow_service.to_dict()) == example_workflow_service)

    def test_example_workflow_config(self, example_workflow_config: WorkflowsConfig) -> None:
        """
        Tests creation and dict conversion of the WorkflowsConfig DAO

        :param example_workflow_config: an example WorkflowsConfig
        :return: None
        """
        assert isinstance(example_workflow_config.to_dict(), dict)
        assert isinstance(WorkflowsConfig.from_dict(example_workflow_config.to_dict()), WorkflowsConfig)
        assert (WorkflowsConfig.from_dict(example_workflow_config.to_dict()).to_dict()
                == example_workflow_config.to_dict())
        assert (WorkflowsConfig.from_dict(example_workflow_config.to_dict()) == example_workflow_config)



from typing import Tuple, Union, Dict, Any
import csle_collector.constants.constants as constants
from csle_collector.docker_stats_manager.dao.docker_stats import DockerStats
from csle_collector.docker_stats_manager.docker_stats_manager_pb2 import DockerStatsMonitorDTO


class DockerStatsUtil:
    """
    Utility class for managing docker container statistics
    """

    @staticmethod
    def calculate_cpu_percent(stats_dict: Dict[str, Any]) -> float:
        """
        Calculates the CPU utilization percentage

        :param stats_dict: the stats dict from the Docker API
        :return: the CPU percentage
        """
        cpu_percent = 0.0
        cpu_usage = stats_dict[constants.DOCKER_STATS.CPU_STATS][constants.DOCKER_STATS.CPU_USAGE]
        precpu_usage = stats_dict[constants.DOCKER_STATS.PRECPU_STATS][constants.DOCKER_STATS.CPU_USAGE]
        cpu_count = len(cpu_usage[constants.DOCKER_STATS.PERCPU_USAGE])
        cpu_delta = (float(cpu_usage[constants.DOCKER_STATS.TOTAL_USAGE]) -
                     float(precpu_usage[constants.DOCKER_STATS.TOTAL_USAGE]))
        system_delta = (float(stats_dict[constants.DOCKER_STATS.CPU_STATS][constants.DOCKER_STATS.SYSTEM_CPU_USAGE]) -
                        float(stats_dict[constants.DOCKER_STATS.PRECPU_STATS][constants.DOCKER_STATS.SYSTEM_CPU_USAGE]))
        if system_delta > 0.0:
            cpu_percent = cpu_delta / system_delta * 100.0 * cpu_count
        return cpu_percent

    @staticmethod
    def calculate_cpu_percent2(stats_dict: Dict[str, Any], previous_cpu: float, previous_system: float) \
            -> Tuple[Union[float, Any], float, float]:
        """
        Calculates the CPU utilization percentage when precpu is broken in later versions of Docker

        :param stats_dict: the stats dict from the Docker API
        :param previous_cpu: previous cpu percentage
        :param previous_system: previous system percentage
        :return: the CPU percentage
        """
        cpu_percent = 0.0
        cpu_usage = stats_dict[constants.DOCKER_STATS.CPU_STATS][constants.DOCKER_STATS.CPU_USAGE]
        cpu_total = float(cpu_usage[constants.DOCKER_STATS.TOTAL_USAGE])
        cpu_delta = cpu_total - previous_cpu
        cpu_system = float(stats_dict[constants.DOCKER_STATS.CPU_STATS][constants.DOCKER_STATS.SYSTEM_CPU_USAGE])
        system_delta = cpu_system - previous_system
        online_cpus = 1
        if constants.DOCKER_STATS.ONLINE_CPUS in stats_dict[constants.DOCKER_STATS.CPU_STATS]:
            online_cpus = stats_dict[constants.DOCKER_STATS.CPU_STATS][constants.DOCKER_STATS.ONLINE_CPUS]
        else:
            if constants.DOCKER_STATS.CPU_USAGE in stats_dict[constants.DOCKER_STATS.CPU_STATS]:
                if (constants.DOCKER_STATS.PERCPU_USAGE in
                        stats_dict[constants.DOCKER_STATS.CPU_STATS][constants.DOCKER_STATS.CPU_USAGE]):
                    cpu_usage = stats_dict[constants.DOCKER_STATS.CPU_STATS][constants.DOCKER_STATS.CPU_USAGE]
                    online_cpus = len(cpu_usage[constants.DOCKER_STATS.PERCPU_USAGE])
        if system_delta > 0.0:
            cpu_percent = (cpu_delta / system_delta) * online_cpus * 100.0
        return cpu_percent, cpu_system, cpu_total

    @staticmethod
    def calculate_blkio_mb(stats_dict: Dict[str, Any]) -> Tuple[float, float]:
        """
        :param stats_dict: the stats dict from the Docker API
        :return: (read_mb, wrote_mb)
        """
        bytes_stats = DockerStatsUtil.graceful_chain_get(stats_dict, constants.DOCKER_STATS.BLKIO_STATS,
                                                         constants.DOCKER_STATS.IO_SERVICE_BYTES_RECURSIVE)
        if not bytes_stats:
            return 0.0, 0.0
        r = 0.0
        w = 0.0
        for s in bytes_stats:
            if s[constants.DOCKER_STATS.OP] == constants.DOCKER_STATS.READ:
                r += s[constants.DOCKER_STATS.VALUE]
            elif s[constants.DOCKER_STATS.OP] == constants.DOCKER_STATS.WRITE:
                w += s[constants.DOCKER_STATS.VALUE]
        r = round(r / 1000000, 1)
        w = round(w / 1000000, 1)
        return r, w

    @staticmethod
    def calculate_network_mb(stats_dict: Dict[str, Any]) -> Tuple[float, float]:
        """
        :param stats_dict: the stats dict from the Docker API
        :return: (received_mb, transceived_mb)
        """
        networks = DockerStatsUtil.graceful_chain_get(stats_dict, constants.DOCKER_STATS.NETWORKS)
        if not networks:
            return 0.0, 0.0
        r = 0.0
        t = 0.0
        for if_name, data in networks.items():
            r += data[constants.DOCKER_STATS.RX_BYTES]
            t += data[constants.DOCKER_STATS.TX_BYTES]
        r = round(r / 1000000, 1)
        t = round(t / 1000000, 1)
        return r, t

    @staticmethod
    def graceful_chain_get(stats_dict: Dict[str, Any], *args, default=None):
        """
        Wrapper to handle errors

        :param stats_dict: stats dict to handle
        :param args: extra arguments
        :param default: default return in case of errors
        :return: default or parsed value
        """
        t = stats_dict
        for a in args:
            try:
                t = t[a]
            except (KeyError, ValueError, TypeError):
                return default
        return t

    @staticmethod
    def parse_stats(stats_dict: Dict[str, Any], container: str) -> DockerStats:
        """
        Parses a stats dict into a DockerStats object

        :param stats_dict: the dict to parse
        :param container: the container that the stats concerns
        :return: the parsed DockerStats object
        """
        cpu_total = 0.0
        cpu_system = 0.0
        blk_read, blk_write = DockerStatsUtil.calculate_blkio_mb(stats_dict)
        net_r, net_w = DockerStatsUtil.calculate_network_mb(stats_dict)
        mem_current = stats_dict[constants.DOCKER_STATS.MEMORY_STATS][constants.DOCKER_STATS.USAGE]
        mem_current = round(mem_current / 1000000, 1)
        mem_total = stats_dict[constants.DOCKER_STATS.MEMORY_STATS][constants.DOCKER_STATS.LIMIT]
        mem_total = round(mem_total / 1000000, 1)
        try:
            cpu_percent, cpu_system, cpu_total = DockerStatsUtil.calculate_cpu_percent2(stats_dict, cpu_total,
                                                                                        cpu_system)
        except KeyError:
            cpu_percent = DockerStatsUtil.calculate_cpu_percent(stats_dict)

        parsed_stats_dict = {
            constants.DOCKER_STATS.PIDS: stats_dict[constants.DOCKER_STATS.PIDS_STATS][constants.DOCKER_STATS.CURRENT],
            constants.DOCKER_STATS.TIMESTAMP: stats_dict[constants.DOCKER_STATS.READ.lower()],
            constants.DOCKER_STATS.CPU_PERCENT: cpu_percent,
            constants.DOCKER_STATS.MEM_CURRENT: mem_current,
            constants.DOCKER_STATS.MEM_TOTAL: mem_total,
            constants.DOCKER_STATS.MEM_PERCENT: (mem_current / mem_total) * 100.0,
            constants.DOCKER_STATS.BLK_READ: blk_read,
            constants.DOCKER_STATS.BLK_WRITE: blk_write,
            constants.DOCKER_STATS.NET_RX: net_r,
            constants.DOCKER_STATS.NET_TX: net_w,
            constants.DOCKER_STATS.CONTAINER_NAME: container
        }
        return DockerStats.from_dict(parsed_stats_dict)

    @staticmethod
    def docker_stats_monitor_dto_to_dict(docker_stats_monitor_dto: DockerStatsMonitorDTO) -> Dict[str, Any]:
        """
        Converts a docker_stats_monitor_dto to a dict

        :param docker_stats_monitor_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d: Dict[str, Any] = {}
        d["num_monitors"] = docker_stats_monitor_dto.num_monitors
        emulations = []
        for em in docker_stats_monitor_dto.emulations:
            emulations.append(str(em))
        d["emulations"] = emulations
        executions = []
        for exec in docker_stats_monitor_dto.emulation_executions:
            executions.append(int(exec))
        d["emulation_executions"] = executions
        return d

    @staticmethod
    def docker_stats_monitor_dto_from_dict(d: Dict[str, Any]) -> DockerStatsMonitorDTO:
        """
        Converts a dict representation of a DockerStatsMonitorDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        docker_stats_monitor_dto = DockerStatsMonitorDTO(
            num_monitors=d["num_monitors"],
            emulations=d["emulations"],
            emulation_executions=d["emulation_executions"]
        )
        return docker_stats_monitor_dto

    @staticmethod
    def docker_stats_monitor_dto_empty() -> DockerStatsMonitorDTO:
        """
        :return: an empty DockerStatsMonitorDTO
        """
        docker_stats_monitor_dto = DockerStatsMonitorDTO()
        docker_stats_monitor_dto.num_monitors = 0
        return docker_stats_monitor_dto



from typing import Any, List
import pytest
import pytest_mock
from csle_collector.kafka_manager.kafka_manager_pb2 import KafkaDTO
from csle_collector.kafka_manager.kafka_manager import KafkaManagerServicer
from csle_collector.kafka_manager.kafka_manager_util import KafkaManagerUtil
import csle_collector.kafka_manager.query_kafka_server
import csle_collector.constants.constants as constants


class TestKafkaManagerSuite:
    """
    Test suite for kafka_manager.py
    """

    @pytest.fixture(scope='module')
    def grpc_add_to_server(self) -> Any:
        """
        Necessary fixture for pytest-grpc

        :return: the add_servicer_to_server function
        """
        from csle_collector.kafka_manager.kafka_manager_pb2_grpc import add_KafkaManagerServicer_to_server
        return add_KafkaManagerServicer_to_server

    @pytest.fixture(scope='module')
    def grpc_servicer(self) -> KafkaManagerServicer:
        """
        Necessary fixture for pytest-grpc

        :return: the kafka manager servicer
        """
        return KafkaManagerServicer()

    @pytest.fixture(scope='module')
    def grpc_stub_cls(self, grpc_channel):
        """
        Necessary fixture for pytest-grpc

        :param grpc_channel: the grpc channel for testing
        :return: the stub to the service
        """
        from csle_collector.kafka_manager.kafka_manager_pb2_grpc import KafkaManagerStub
        return KafkaManagerStub

    def test_getKafkaStatus(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the getKafkaStatus grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        running = True
        topics = ["topic1", "topic2"]
        mocker.patch('csle_collector.kafka_manager.kafka_manager.KafkaManagerServicer._get_kafka_status_and_topics',
                     return_value=(running, topics))
        response: KafkaDTO = csle_collector.kafka_manager.query_kafka_server.get_kafka_status(stub=grpc_stub)
        assert response.running == running
        assert response.topics == topics

    def test_stopKafka(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the stopKafka grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        running = False
        topics: List[str] = []
        mocker.patch('os.system', return_value=True)
        response: KafkaDTO = csle_collector.kafka_manager.query_kafka_server.stop_kafka(stub=grpc_stub)
        assert response.running == running
        assert response.topics == topics

    def test_startKafka(self, grpc_stub, mocker: pytest_mock.MockFixture) -> None:
        """
        Tests the startKafka grpc

        :param grpc_stub: the stub for the GRPC server to make the request to
        :param mocker: the mocker object to mock functions with external dependencies
        :return: None
        """
        running = True
        topics: List[str] = []
        mocker.patch('os.system', return_value=True)
        response: KafkaDTO = csle_collector.kafka_manager.query_kafka_server.start_kafka(stub=grpc_stub)
        assert response.running == running
        assert response.topics == topics

    def test_hours_to_ms(self) -> None:
        """
        Tests the hours_to_ms function

        :return: None
        """
        assert KafkaManagerUtil.hours_to_ms(1 / (60 * 60)) == 1000
        assert KafkaManagerUtil.hours_to_ms(1 / (60)) == 60000
        assert KafkaManagerUtil.hours_to_ms(1) == 3.6e6

    def test_kafka_dto_empty(self) -> None:
        """
        Tests the kafka_dto_empty function

        :return: None
        """
        dto: KafkaDTO = KafkaManagerUtil.kafka_dto_empty()
        assert not dto.running
        assert dto.topics == []

    def test_kafka_dto_from_dict(self) -> None:
        """
        Tests the kafka_dto_from_dict function

        :return: None
        """
        dicts = [{constants.DICT_PROPERTIES.RUNNING: True, constants.DICT_PROPERTIES.TOPICS: ["topic1", "topic2"]},
                 {constants.DICT_PROPERTIES.RUNNING: False, constants.DICT_PROPERTIES.TOPICS: []},
                 {constants.DICT_PROPERTIES.RUNNING: False, constants.DICT_PROPERTIES.TOPICS: ["topic3"]}]
        for d in dicts:
            dto: KafkaDTO = KafkaManagerUtil.kafka_dto_from_dict(d=d)
            assert dto.running == d[constants.DICT_PROPERTIES.RUNNING]
            assert dto.topics == d[constants.DICT_PROPERTIES.TOPICS]

    def test_kafka_dto_to_dict(self) -> None:
        """
        Tests the kafka_dto_to_dict function

        :return: None
        """
        dtos = [KafkaDTO(running=True, topics=["topic1", "topic2"]), KafkaDTO(running=False, topics=[]),
                KafkaDTO(running=True, topics=["topic3"])]
        for dto in dtos:
            d = KafkaManagerUtil.kafka_dto_to_dict(kafka_dto=dto)
            assert dto.running == d[constants.DICT_PROPERTIES.RUNNING]
            assert dto.topics == d[constants.DICT_PROPERTIES.TOPICS]



import threading
import time
from confluent_kafka import Producer
import logging
from csle_collector.ossec_ids_manager.ossec_ids_manager_util import OSSecManagerUtil
import csle_collector.constants.constants as constants


class OSSecIdsMonitorThread(threading.Thread):
    """
    Thread that collects the OSSEC IDS statistics and pushes it to Kafka periodically
    """

    def __init__(self, kafka_ip: str, kafka_port: int, ip: str, hostname: str, log_file_path: str,
                 time_step_len_seconds: int):
        """
        Initializes the thread

        :param kafka_ip: IP of the Kafka server to push to
        :param kafka_port: port of the Kafka server to push to
        :param ip: ip of the server we are pushing from
        :param hostname: hostname of the server we are pushing from
        :param log_file_path: path to the IDS log
        :param time_step_len_seconds: the length of a timestep
        """
        threading.Thread.__init__(self)
        self.kafka_ip = kafka_ip
        self.kafka_port = kafka_port
        self.ip = ip
        self.hostname = hostname
        self.log_file_path = log_file_path
        self.latest_ts = time.time()
        self.time_step_len_seconds = time_step_len_seconds
        self.conf = {constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.kafka_ip}:{self.kafka_port}",
                     constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.producer = Producer(**self.conf)
        self.running = True
        logging.info("OSSEC IDSMonitor thread started successfully")

    def run(self) -> None:
        """
        Main loop of the thread. Parses the IDS log and pushes it to Kafka periodically

        :return: None
        """
        while self.running:
            time.sleep(self.time_step_len_seconds)
            alert_counters = OSSecManagerUtil.read_ossec_ids_data(self.latest_ts)
            record = alert_counters.to_kafka_record(ip=self.ip)
            self.producer.produce(constants.KAFKA_CONFIG.OSSEC_IDS_LOG_TOPIC_NAME, record)
            self.producer.poll(0)
            self.latest_ts = time.time()



from typing import List, Dict, Any, Union
from csle_collector.client_manager.dao.workflow_markov_chain import WorkflowMarkovChain
from csle_collector.client_manager.dao.workflow_service import WorkflowService
import csle_collector.client_manager.client_manager_pb2
from csle_base.json_serializable import JSONSerializable
from csle_base.grpc_serializable import GRPCSerializable


class WorkflowsConfig(JSONSerializable, GRPCSerializable):
    """
    Workflows configuration
    """

    def __init__(self, workflow_markov_chains: List[WorkflowMarkovChain], workflow_services: List[WorkflowService]) \
            -> None:
        """
        Initializes the object

        :param workflow_markov_chains: the workflow Markov chains
        :param workflow_services: the workflow services
        """
        self.workflow_markov_chains = workflow_markov_chains
        self.workflow_services = workflow_services

    def get_workflow_service(self, id: int) -> Union[WorkflowService, None]:
        """
        Gets the workflow service with a specific id

        :param id: the id of the service
        :return: the service or None
        """
        for service in self.workflow_services:
            if service.id == id:
                return service
        return None

    def get_workflow_mc(self, id: int) -> Union[WorkflowMarkovChain, None]:
        """
        Gets the workflow Markov chain with a specific id

        :param id: the id of the MC
        :return: the MC or None
        """
        for mc in self.workflow_markov_chains:
            if mc.id == id:
                return mc
        return None

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "WorkflowsConfig":
        """
        Converts a dict representation to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = WorkflowsConfig(
            workflow_services=list(map(lambda x: WorkflowService.from_dict(x), d["workflow_services"])),
            workflow_markov_chains=list(map(lambda x: WorkflowMarkovChain.from_dict(x), d["workflow_markov_chains"]))
        )
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d = {}
        d["workflow_services"] = list(map(lambda x: x.to_dict(), self.workflow_services))
        d["workflow_markov_chains"] = list(map(lambda x: x.to_dict(), self.workflow_markov_chains))
        return d

    @staticmethod
    def from_json_file(json_file_path: str) -> "WorkflowsConfig":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return WorkflowsConfig.from_dict(json.loads(json_str))

    def copy(self) -> "WorkflowsConfig":
        """
        :return: a copy of the DTO
        """
        return WorkflowsConfig.from_dict(self.to_dict())

    def create_execution_config(self, ip_first_octet: int) -> "WorkflowsConfig":
        """
        Creates a new config for an execution

        :param ip_first_octet: the first octet of the IP of the new execution
        :return: the new config
        """
        config = self.copy()
        config.workflow_services = list(map(lambda x: x.create_execution_config(ip_first_octet=ip_first_octet),
                                            config.workflow_services))
        return config

    def to_grpc_object(self) -> csle_collector.client_manager.client_manager_pb2.WorkflowsConfigDTO:
        """
        :return: a GRPC serializable version of the object
        """
        mcs = list(map(lambda x: x.to_grpc_object(), self.workflow_markov_chains))
        services = list(map(lambda x: x.to_grpc_object(), self.workflow_services))
        return csle_collector.client_manager.client_manager_pb2.WorkflowsConfigDTO(
            workflow_markov_chains=mcs, workflow_services=services)

    @staticmethod
    def from_grpc_object(obj: csle_collector.client_manager.client_manager_pb2.WorkflowsConfigDTO) \
            -> "WorkflowsConfig":
        """
        Instantiates the object from a GRPC DTO

        :param obj: the object to instantiate from
        :return: the instantiated object
        """
        mcs = list(map(lambda x: WorkflowMarkovChain.from_grpc_object(x), obj.workflow_markov_chains))
        services = list(map(lambda x: WorkflowService.from_grpc_object(x), obj.workflow_services))
        return WorkflowsConfig(workflow_markov_chains=mcs, workflow_services=services)

    def commands(self) -> List[str]:
        """
        Gets the list of all commands across all services

        :return: the list of commands
        """
        commands: List[str] = []
        for w in self.workflow_services:
            commands = commands + w.get_commands()
        return commands

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"Workflows configuration; markov chains: {list(map(lambda x: str(x), self.workflow_markov_chains))}, " \
               f"workflow services: {list(map(lambda x: str(x), self.workflow_services))}"



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class StartLogstashMsg(google.protobuf.message.Message):
    """Message that the client sends to stop logstash"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartLogstashMsg = StartLogstashMsg

@typing_extensions.final
class StartKibanaMsg(google.protobuf.message.Message):
    """Message that the client sends to stop kibana"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartKibanaMsg = StartKibanaMsg

@typing_extensions.final
class StartElasticMsg(google.protobuf.message.Message):
    """Message that the client sends to stop elasticsearch"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartElasticMsg = StartElasticMsg

@typing_extensions.final
class StopLogstashMsg(google.protobuf.message.Message):
    """Message that the client sends to stop logstash"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopLogstashMsg = StopLogstashMsg

@typing_extensions.final
class StopKibanaMsg(google.protobuf.message.Message):
    """Message that the client sends to stop kibana"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopKibanaMsg = StopKibanaMsg

@typing_extensions.final
class StopElasticMsg(google.protobuf.message.Message):
    """Message that the client sends to stop elasticsearch"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopElasticMsg = StopElasticMsg

@typing_extensions.final
class StopElkMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the ELK stack"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopElkMsg = StopElkMsg

@typing_extensions.final
class StartElkMsg(google.protobuf.message.Message):
    """Message that the client sends to start the ELK stack"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartElkMsg = StartElkMsg

@typing_extensions.final
class GetElkStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the elk status"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetElkStatusMsg = GetElkStatusMsg

@typing_extensions.final
class ElkDTO(google.protobuf.message.Message):
    """Message that the server returns when requested by the client, contains info about the ELK stack"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ELASTICRUNNING_FIELD_NUMBER: builtins.int
    KIBANARUNNING_FIELD_NUMBER: builtins.int
    LOGSTASHRUNNING_FIELD_NUMBER: builtins.int
    elasticRunning: builtins.bool
    kibanaRunning: builtins.bool
    logstashRunning: builtins.bool
    def __init__(
        self,
        *,
        elasticRunning: builtins.bool = ...,
        kibanaRunning: builtins.bool = ...,
        logstashRunning: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elasticRunning", b"elasticRunning", "kibanaRunning", b"kibanaRunning", "logstashRunning", b"logstashRunning"]) -> None: ...

global___ElkDTO = ElkDTO



from csle_collector.snort_ids_manager.dao.snort_ids_alert import SnortIdsAlert
from csle_collector.snort_ids_manager.dao.snort_ids_alert_counters import SnortIdsAlertCounters
from csle_collector.snort_ids_manager.dao.snort_ids_ip_alert_counters import SnortIdsIPAlertCounters
from csle_collector.snort_ids_manager.dao.snort_ids_rule_counters import SnortIdsRuleCounters
from csle_collector.snort_ids_manager.dao.snort_ids_fast_log_alert import SnortIdsFastLogAlert


class TestSnortIDSManagerDaoSuite:
    """
    Test suite for datasets data access objects (DAOs) in snort_ide_manager
    """

    def test_snort_ids_alert(self, example_snort_ids_alert: SnortIdsAlert) -> None:
        """
        Tests creation and dict conversion of the SnortIdsAlert DAO

        :param example_snort_ids_alert: an example SnortIdsAlert
        :return: None
        """
        assert isinstance(example_snort_ids_alert.to_dict(), dict)
        assert isinstance(SnortIdsAlert.from_dict(example_snort_ids_alert.to_dict()),
                          SnortIdsAlert)
        assert (SnortIdsAlert.from_dict(example_snort_ids_alert.to_dict()).to_dict()
                == example_snort_ids_alert.to_dict())
        assert (SnortIdsAlert.from_dict(example_snort_ids_alert.to_dict())
                == example_snort_ids_alert)

    def test_snort_ids_alert_counters(self, example_snort_ids_alert_counters: SnortIdsAlertCounters) -> None:
        """
        Tests creation and dict conversion of the SnortIdsAlertCounters DAO

        :param example_snort_ids_alert_counters: an example SnortIdsAlertCounters
        :return: None
        """
        assert isinstance(example_snort_ids_alert_counters.to_dict(), dict)
        assert isinstance(SnortIdsAlertCounters.from_dict(example_snort_ids_alert_counters.to_dict()),
                          SnortIdsAlertCounters)
        assert (SnortIdsAlertCounters.from_dict(example_snort_ids_alert_counters.to_dict()).to_dict()
                == example_snort_ids_alert_counters.to_dict())
        assert (SnortIdsAlertCounters.from_dict(example_snort_ids_alert_counters.to_dict())
                == example_snort_ids_alert_counters)

    def test_snort_ids_ip_alert_counters(self, example_snort_ids_ip_alert_counters: SnortIdsIPAlertCounters) -> None:
        """
        Tests creation and dict conversion of the SnortIdsIPAlertCounters DAO

        :param example_snort_ids_ip_alert_counters: an example SnortIdsIPAlertCounters
        :return: None
        """
        assert isinstance(example_snort_ids_ip_alert_counters.to_dict(), dict)
        assert isinstance(SnortIdsIPAlertCounters.from_dict(example_snort_ids_ip_alert_counters.to_dict()),
                          SnortIdsIPAlertCounters)
        assert (SnortIdsIPAlertCounters.from_dict(example_snort_ids_ip_alert_counters.to_dict()).to_dict()
                == example_snort_ids_ip_alert_counters.to_dict())
        assert (SnortIdsIPAlertCounters.from_dict(example_snort_ids_ip_alert_counters.to_dict())
                == example_snort_ids_ip_alert_counters)

    def test_snort_ids_rule_counters(self, example_snort_ids_rule_counters: SnortIdsRuleCounters) -> None:
        """
        Tests creation and dict conversion of the SnortIdsRuleCounters DAO

        :param example_snort_ids_rule_counters: an example SnortIdsRuleCounters
        :return: None
        """
        assert isinstance(example_snort_ids_rule_counters.to_dict(), dict)
        assert isinstance(SnortIdsRuleCounters.from_dict(example_snort_ids_rule_counters.to_dict()),
                          SnortIdsRuleCounters)
        assert (SnortIdsRuleCounters.from_dict(example_snort_ids_rule_counters.to_dict()).to_dict()
                == example_snort_ids_rule_counters.to_dict())
        assert (SnortIdsRuleCounters.from_dict(example_snort_ids_rule_counters.to_dict())
                == example_snort_ids_rule_counters)

    def test_snort_ids_log_alert(self, example_snort_ids_fast_alert: SnortIdsFastLogAlert) -> None:
        """
        Tests creation and dict conversion of the SnortIdsFastLogAlert DAO

        :param example_snort_ids_fast_alert: an example SnortIdsFastLogAlert
        :return: None
        """
        assert isinstance(example_snort_ids_fast_alert.to_dict(), dict)
        assert isinstance(SnortIdsFastLogAlert.from_dict(example_snort_ids_fast_alert.to_dict()),
                          SnortIdsFastLogAlert)
        assert (SnortIdsFastLogAlert.from_dict(example_snort_ids_fast_alert.to_dict()).to_dict()
                == example_snort_ids_fast_alert.to_dict())
        assert (SnortIdsFastLogAlert.from_dict(example_snort_ids_fast_alert.to_dict())
                == example_snort_ids_fast_alert)



import datetime
from csle_base.json_serializable import JSONSerializable
from typing import Dict, Any


class SuccessfulLogin(JSONSerializable):
    """
    DTO Class representing a parsed successful login event from a server in the emulation
    """

    def __init__(self):
        """
        Initializes the DTO
        """
        self.timestamp = None
        self.ip = ""
        self.user = ""

    @staticmethod
    def parse_from_str(login_attempt_str: str, year: int):
        """
        Parses a login event from a string

        :param login_attempt_str: the string to parse
        :param year: the current year
        :return: the parsed login event
        """
        successful_login_attempt_dto = SuccessfulLogin()
        parts = login_attempt_str.split()
        if len(parts) > 6:
            successful_login_attempt_dto.user = parts[0]
            successful_login_attempt_dto.ip = parts[2]
            date_str = " ".join(parts[3:7])
            date_str = str(year) + " " + date_str
            successful_login_attempt_dto.timestamp = \
                datetime.datetime.strptime(date_str, '%Y %a %b %d %H:%M').timestamp()
        return successful_login_attempt_dto

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return "ip:{}, user:{}, timestamp:{}".format(self.ip, self.user, self.timestamp)

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Any] = {}
        d["timestamp"] = self.timestamp
        d["ip"] = self.ip
        d["user"] = self.user
        return d

    @staticmethod
    def from_dict(parsed_stats_dict: Dict[str, Any]) -> "SuccessfulLogin":
        """
        Parses a FailedLoginAttempt object from a dict

        :param parsed_stats_dict: the dict to parse
        :return: the parsed FailedLoginAttempt object
        """
        successful_login_dto = SuccessfulLogin()
        successful_login_dto.timestamp = parsed_stats_dict["timestamp"]
        successful_login_dto.ip = parsed_stats_dict["ip"]
        successful_login_dto.user = parsed_stats_dict["user"]
        return successful_login_dto

    @staticmethod
    def from_json_file(json_file_path: str) -> "SuccessfulLogin":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return SuccessfulLogin.from_dict(json.loads(json_str))



import random
from typing import List, Dict, Any
import numpy as np
from scipy.stats import expon
from csle_collector.client_manager.dao.arrival_config import ArrivalConfig
from csle_collector.client_manager.dao.eptmp_arrival_config import EPTMPArrivalConfig
from csle_collector.client_manager.dao.spiking_arrival_config import SpikingArrivalConfig
from csle_collector.client_manager.dao.sine_arrival_config import SineArrivalConfig
from csle_collector.client_manager.dao.piece_wise_constant_arrival_config import PieceWiseConstantArrivalConfig
from csle_collector.client_manager.dao.constant_arrival_config import ConstantArrivalConfig
from csle_collector.client_manager.dao.client_arrival_type import ClientArrivalType
from csle_collector.client_manager.dao.workflows_config import WorkflowsConfig
import csle_collector.client_manager.client_manager_pb2
from csle_base.json_serializable import JSONSerializable
from csle_base.grpc_serializable import GRPCSerializable


class Client(JSONSerializable, GRPCSerializable):
    """
    A client, which is characterized by its arrival process and its workflow distribution.
    """

    def __init__(self, id: int, workflow_distribution: List[float], arrival_config: ArrivalConfig, mu: float = 4,
                 exponential_service_time: bool = False) -> None:
        """
        Initializes the object

        :param id: the client id
        :param mu: the mean service time if exponential service times are used (otherwise MC is used)
        :param exponential_service_time: boolean flag indicating whether exponential service times should be used
        :param arrival_config: the arrival process configuration of the client
        :param workflow_distribution: the workflow distribution of the client
        """
        self.id = id
        self.mu = mu
        self.exponential_service_time = exponential_service_time
        self.arrival_config = arrival_config
        assert round(sum(workflow_distribution), 2) == 1
        assert self.arrival_config is not None
        self.workflow_distribution = workflow_distribution

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "Client":
        """
        Converts a dict representation to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        if d["arrival_config"]["client_arrival_type"] == ClientArrivalType.CONSTANT.value:
            arrival_config = ConstantArrivalConfig.from_dict(d["arrival_config"])
        elif d["arrival_config"]["client_arrival_type"] == ClientArrivalType.EPTMP.value:
            arrival_config = EPTMPArrivalConfig.from_dict(d["arrival_config"])
        elif d["arrival_config"]["client_arrival_type"] == ClientArrivalType.SPIKING.value:
            arrival_config = SpikingArrivalConfig.from_dict(d["arrival_config"])
        elif d["arrival_config"]["client_arrival_type"] == ClientArrivalType.SINE_MODULATED.value:
            arrival_config = SineArrivalConfig.from_dict(d["arrival_config"])
        elif d["arrival_config"]["client_arrival_type"] == ClientArrivalType.PIECE_WISE_CONSTANT.value:
            arrival_config = PieceWiseConstantArrivalConfig.from_dict(d["arrival_config"])
        else:
            raise ValueError("Arrival config not recognized")
        obj = Client(
            arrival_config=arrival_config, id=d["id"], mu=d["mu"],
            exponential_service_time=d["exponential_service_time"], workflow_distribution=d["workflow_distribution"]
        )
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Any] = {}
        d["arrival_config"] = self.arrival_config.to_dict()
        d["id"] = self.id
        d["mu"] = self.mu
        d["exponential_service_time"] = self.exponential_service_time
        d["workflow_distribution"] = self.workflow_distribution
        return d

    @staticmethod
    def from_json_file(json_file_path: str) -> "Client":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return Client.from_dict(json.loads(json_str))

    def copy(self) -> "Client":
        """
        :return: a copy of the DTO
        """
        return Client.from_dict(self.to_dict())

    def to_grpc_object(self) -> csle_collector.client_manager.client_manager_pb2.ClientDTO:
        """
        :return: a GRPC serializable version of the object
        """
        constant_arrival_config = None
        sine_arrival_config = None
        spiking_arrival_config = None
        piece_wise_constant_arrival_config = None
        eptmp_arrival_config = None
        if self.arrival_config.client_arrival_type == ClientArrivalType.CONSTANT:
            constant_arrival_config = self.arrival_config.to_grpc_object()
        elif self.arrival_config.client_arrival_type == ClientArrivalType.SINE_MODULATED:
            sine_arrival_config = self.arrival_config.to_grpc_object()
        elif self.arrival_config.client_arrival_type == ClientArrivalType.SPIKING:
            spiking_arrival_config = self.arrival_config.to_grpc_object()
        elif self.arrival_config.client_arrival_type == ClientArrivalType.PIECE_WISE_CONSTANT:
            piece_wise_constant_arrival_config = self.arrival_config.to_grpc_object()
        elif self.arrival_config.client_arrival_type == ClientArrivalType.EPTMP:
            eptmp_arrival_config = self.arrival_config.to_grpc_object()
        else:
            raise ValueError(f"Client arrival type: {self.arrival_config.client_arrival_type} not recognized")
        return csle_collector.client_manager.client_manager_pb2.ClientDTO(
            id=self.id, workflow_distribution=self.workflow_distribution, mu=self.mu,
            exponential_service_time=self.exponential_service_time, constant_arrival_config=constant_arrival_config,
            sine_arrival_config=sine_arrival_config, spiking_arrival_config=spiking_arrival_config,
            piece_wise_constant_arrival_config=piece_wise_constant_arrival_config,
            eptmp_arrival_config=eptmp_arrival_config, arrival_type=self.arrival_config.client_arrival_type.value)

    @staticmethod
    def from_grpc_object(obj: csle_collector.client_manager.client_manager_pb2.ClientDTO) -> "Client":
        """
        Instantiates the object from a GRPC DTO

        :param obj: the object to instantiate from
        :return: the instantiated object
        """
        arrival_type = ClientArrivalType(obj.arrival_type)
        if arrival_type.value == ClientArrivalType.EPTMP.value:
            arrival_config = EPTMPArrivalConfig.from_grpc_object(obj.eptmp_arrival_config)
        elif arrival_type.value == ClientArrivalType.SINE_MODULATED.value:
            arrival_config = SineArrivalConfig.from_grpc_object(obj.sine_arrival_config)
        elif arrival_type.value == ClientArrivalType.SPIKING.value:
            arrival_config = SpikingArrivalConfig.from_grpc_object(obj.spiking_arrival_config)
        elif arrival_type.value == ClientArrivalType.PIECE_WISE_CONSTANT.value:
            arrival_config = PieceWiseConstantArrivalConfig.from_grpc_object(obj.piece_wise_constant_arrival_config)
        elif arrival_type.value == ClientArrivalType.CONSTANT.value:
            arrival_config = ConstantArrivalConfig.from_grpc_object(obj.constant_arrival_config)
        else:
            raise ValueError(f"Client arrival type: {arrival_type} not recognized")
        workflow_distribution = list(map(lambda x: float(x), obj.workflow_distribution))
        return Client(id=obj.id, workflow_distribution=workflow_distribution, mu=obj.mu,
                      exponential_service_time=obj.exponential_service_time, arrival_config=arrival_config)

    def generate_commands(self, workflows_config: WorkflowsConfig) -> List[str]:
        """
        Generate the commands for the client

        :param workflows_config: the workflows configuration
        :return: sampled list of commands for the client
        """
        commands: List[str] = []
        w = np.random.choice(np.arange(0, len(workflows_config.workflow_services)), p=self.workflow_distribution)
        mc = workflows_config.get_workflow_mc(id=w)
        if mc is None:
            raise ValueError(f"Workflow not recognized: {w}")
        s = mc.initial_state
        service_time = 0
        if self.exponential_service_time:
            service_time = expon.rvs(scale=(self.mu), loc=0, size=1)[0]
        done = False
        while not done:
            if not self.exponential_service_time:
                done = s == len(workflows_config.workflow_services)
            else:
                done = len(commands) > service_time
            if not done:
                service = workflows_config.get_workflow_service(id=s)
                if service is None:
                    raise ValueError(f"Service with id: {s} not found, available service ids: "
                                     f"{list(map(lambda x: x.id, workflows_config.workflow_services))}")
                service_cmds = service.get_commands()
                commands.append(random.choice(service_cmds))
                s_prime = mc.step_forward()
                if not self.exponential_service_time or s_prime != len(workflows_config.workflow_services):
                    s = s_prime
        mc.reset()
        return commands

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"id: {self.id}, workflow distribution: {self.workflow_distribution}, mu: {self.mu}, " \
               f"exponential service time: {self.exponential_service_time}, arrival config: {str(self.arrival_config)}"



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class StopClientsMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the clients processes"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopClientsMsg = StopClientsMsg

@typing_extensions.final
class StartClientsMsg(google.protobuf.message.Message):
    """Message that the client sends to start the clients processes"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIME_STEP_LEN_SECONDS_FIELD_NUMBER: builtins.int
    WORKFLOWS_CONFIG_FIELD_NUMBER: builtins.int
    CLIENTS_FIELD_NUMBER: builtins.int
    time_step_len_seconds: builtins.int
    @property
    def workflows_config(self) -> global___WorkflowsConfigDTO: ...
    @property
    def clients(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ClientDTO]: ...
    def __init__(
        self,
        *,
        time_step_len_seconds: builtins.int = ...,
        workflows_config: global___WorkflowsConfigDTO | None = ...,
        clients: collections.abc.Iterable[global___ClientDTO] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["workflows_config", b"workflows_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["clients", b"clients", "time_step_len_seconds", b"time_step_len_seconds", "workflows_config", b"workflows_config"]) -> None: ...

global___StartClientsMsg = StartClientsMsg

@typing_extensions.final
class ClientDTO(google.protobuf.message.Message):
    """ClientDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    WORKFLOW_DISTRIBUTION_FIELD_NUMBER: builtins.int
    CONSTANT_ARRIVAL_CONFIG_FIELD_NUMBER: builtins.int
    SINE_ARRIVAL_CONFIG_FIELD_NUMBER: builtins.int
    SPIKING_ARRIVAL_CONFIG_FIELD_NUMBER: builtins.int
    PIECE_WISE_CONSTANT_ARRIVAL_CONFIG_FIELD_NUMBER: builtins.int
    EPTMP_ARRIVAL_CONFIG_FIELD_NUMBER: builtins.int
    MU_FIELD_NUMBER: builtins.int
    EXPONENTIAL_SERVICE_TIME_FIELD_NUMBER: builtins.int
    ARRIVAL_TYPE_FIELD_NUMBER: builtins.int
    id: builtins.int
    @property
    def workflow_distribution(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def constant_arrival_config(self) -> global___ConstantArrivalConfigDTO: ...
    @property
    def sine_arrival_config(self) -> global___SineArrivalConfigDTO: ...
    @property
    def spiking_arrival_config(self) -> global___SpikingArrivalConfigDTO: ...
    @property
    def piece_wise_constant_arrival_config(self) -> global___PieceWiseConstantArrivalConfigDTO: ...
    @property
    def eptmp_arrival_config(self) -> global___EPTMPArrivalConfigDTO: ...
    mu: builtins.float
    exponential_service_time: builtins.bool
    arrival_type: builtins.int
    def __init__(
        self,
        *,
        id: builtins.int = ...,
        workflow_distribution: collections.abc.Iterable[builtins.float] | None = ...,
        constant_arrival_config: global___ConstantArrivalConfigDTO | None = ...,
        sine_arrival_config: global___SineArrivalConfigDTO | None = ...,
        spiking_arrival_config: global___SpikingArrivalConfigDTO | None = ...,
        piece_wise_constant_arrival_config: global___PieceWiseConstantArrivalConfigDTO | None = ...,
        eptmp_arrival_config: global___EPTMPArrivalConfigDTO | None = ...,
        mu: builtins.float = ...,
        exponential_service_time: builtins.bool = ...,
        arrival_type: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["constant_arrival_config", b"constant_arrival_config", "eptmp_arrival_config", b"eptmp_arrival_config", "piece_wise_constant_arrival_config", b"piece_wise_constant_arrival_config", "sine_arrival_config", b"sine_arrival_config", "spiking_arrival_config", b"spiking_arrival_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["arrival_type", b"arrival_type", "constant_arrival_config", b"constant_arrival_config", "eptmp_arrival_config", b"eptmp_arrival_config", "exponential_service_time", b"exponential_service_time", "id", b"id", "mu", b"mu", "piece_wise_constant_arrival_config", b"piece_wise_constant_arrival_config", "sine_arrival_config", b"sine_arrival_config", "spiking_arrival_config", b"spiking_arrival_config", "workflow_distribution", b"workflow_distribution"]) -> None: ...

global___ClientDTO = ClientDTO

@typing_extensions.final
class GetClientsMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the ClientsDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetClientsMsg = GetClientsMsg

@typing_extensions.final
class StartProducerMsg(google.protobuf.message.Message):
    """Message that the client sends to start the producer thread"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IP_FIELD_NUMBER: builtins.int
    PORT_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_SECONDS_FIELD_NUMBER: builtins.int
    ip: builtins.str
    port: builtins.int
    time_step_len_seconds: builtins.int
    def __init__(
        self,
        *,
        ip: builtins.str = ...,
        port: builtins.int = ...,
        time_step_len_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ip", b"ip", "port", b"port", "time_step_len_seconds", b"time_step_len_seconds"]) -> None: ...

global___StartProducerMsg = StartProducerMsg

@typing_extensions.final
class StopProducerMsg(google.protobuf.message.Message):
    """Message that the client sends to start the producer thread"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopProducerMsg = StopProducerMsg

@typing_extensions.final
class ProbabilityDistributionDTO(google.protobuf.message.Message):
    """ProbabilityDistributionDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROBABILITIES_FIELD_NUMBER: builtins.int
    @property
    def probabilities(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        probabilities: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["probabilities", b"probabilities"]) -> None: ...

global___ProbabilityDistributionDTO = ProbabilityDistributionDTO

@typing_extensions.final
class TransitionMatrixDTO(google.protobuf.message.Message):
    """TransitionMatrixDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROWS_FIELD_NUMBER: builtins.int
    @property
    def rows(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ProbabilityDistributionDTO]: ...
    def __init__(
        self,
        *,
        rows: collections.abc.Iterable[global___ProbabilityDistributionDTO] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["rows", b"rows"]) -> None: ...

global___TransitionMatrixDTO = TransitionMatrixDTO

@typing_extensions.final
class WorkflowMarkovChainDTO(google.protobuf.message.Message):
    """WorkflowMarkovChainDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TRANSITION_MATRIX_FIELD_NUMBER: builtins.int
    INITIAL_STATE_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    @property
    def transition_matrix(self) -> global___TransitionMatrixDTO: ...
    initial_state: builtins.int
    id: builtins.int
    def __init__(
        self,
        *,
        transition_matrix: global___TransitionMatrixDTO | None = ...,
        initial_state: builtins.int = ...,
        id: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["transition_matrix", b"transition_matrix"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id", "initial_state", b"initial_state", "transition_matrix", b"transition_matrix"]) -> None: ...

global___WorkflowMarkovChainDTO = WorkflowMarkovChainDTO

@typing_extensions.final
class WorkflowServiceDTO(google.protobuf.message.Message):
    """WorkflowServiceDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    IPS_FIELD_NUMBER: builtins.int
    COMMANDS_FIELD_NUMBER: builtins.int
    id: builtins.int
    @property
    def ips(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def commands(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NodeCommandsDTO]: ...
    def __init__(
        self,
        *,
        id: builtins.int = ...,
        ips: collections.abc.Iterable[builtins.str] | None = ...,
        commands: collections.abc.Iterable[global___NodeCommandsDTO] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commands", b"commands", "id", b"id", "ips", b"ips"]) -> None: ...

global___WorkflowServiceDTO = WorkflowServiceDTO

@typing_extensions.final
class NodeCommandsDTO(google.protobuf.message.Message):
    """NodeCommandsDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMANDS_FIELD_NUMBER: builtins.int
    @property
    def commands(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        commands: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commands", b"commands"]) -> None: ...

global___NodeCommandsDTO = NodeCommandsDTO

@typing_extensions.final
class WorkflowsConfigDTO(google.protobuf.message.Message):
    """WorkflowServiceDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WORKFLOW_MARKOV_CHAINS_FIELD_NUMBER: builtins.int
    WORKFLOW_SERVICES_FIELD_NUMBER: builtins.int
    @property
    def workflow_markov_chains(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___WorkflowMarkovChainDTO]: ...
    @property
    def workflow_services(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___WorkflowServiceDTO]: ...
    def __init__(
        self,
        *,
        workflow_markov_chains: collections.abc.Iterable[global___WorkflowMarkovChainDTO] | None = ...,
        workflow_services: collections.abc.Iterable[global___WorkflowServiceDTO] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["workflow_markov_chains", b"workflow_markov_chains", "workflow_services", b"workflow_services"]) -> None: ...

global___WorkflowsConfigDTO = WorkflowsConfigDTO

@typing_extensions.final
class ClientsDTO(google.protobuf.message.Message):
    """Message that the server returns when requested by the client"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUM_CLIENTS_FIELD_NUMBER: builtins.int
    CLIENT_PROCESS_ACTIVE_FIELD_NUMBER: builtins.int
    PRODUCER_ACTIVE_FIELD_NUMBER: builtins.int
    CLIENTS_TIME_STEP_LEN_SECONDS_FIELD_NUMBER: builtins.int
    PRODUCER_TIME_STEP_LEN_SECONDS_FIELD_NUMBER: builtins.int
    num_clients: builtins.int
    client_process_active: builtins.bool
    producer_active: builtins.bool
    clients_time_step_len_seconds: builtins.int
    producer_time_step_len_seconds: builtins.int
    def __init__(
        self,
        *,
        num_clients: builtins.int = ...,
        client_process_active: builtins.bool = ...,
        producer_active: builtins.bool = ...,
        clients_time_step_len_seconds: builtins.int = ...,
        producer_time_step_len_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["client_process_active", b"client_process_active", "clients_time_step_len_seconds", b"clients_time_step_len_seconds", "num_clients", b"num_clients", "producer_active", b"producer_active", "producer_time_step_len_seconds", b"producer_time_step_len_seconds"]) -> None: ...

global___ClientsDTO = ClientsDTO

@typing_extensions.final
class ConstantArrivalConfigDTO(google.protobuf.message.Message):
    """ConstantArrivalConfigDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LAMB_FIELD_NUMBER: builtins.int
    lamb: builtins.float
    def __init__(
        self,
        *,
        lamb: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lamb", b"lamb"]) -> None: ...

global___ConstantArrivalConfigDTO = ConstantArrivalConfigDTO

@typing_extensions.final
class SineArrivalConfigDTO(google.protobuf.message.Message):
    """SineArrivalConfigDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LAMB_FIELD_NUMBER: builtins.int
    TIME_SCALING_FACTOR_FIELD_NUMBER: builtins.int
    PERIOD_SCALING_FACTOR_FIELD_NUMBER: builtins.int
    lamb: builtins.float
    time_scaling_factor: builtins.float
    period_scaling_factor: builtins.float
    def __init__(
        self,
        *,
        lamb: builtins.float = ...,
        time_scaling_factor: builtins.float = ...,
        period_scaling_factor: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lamb", b"lamb", "period_scaling_factor", b"period_scaling_factor", "time_scaling_factor", b"time_scaling_factor"]) -> None: ...

global___SineArrivalConfigDTO = SineArrivalConfigDTO

@typing_extensions.final
class SpikingArrivalConfigDTO(google.protobuf.message.Message):
    """SpikingArrivalConfigDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXPONENTS_FIELD_NUMBER: builtins.int
    FACTORS_FIELD_NUMBER: builtins.int
    @property
    def exponents(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def factors(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        exponents: collections.abc.Iterable[builtins.float] | None = ...,
        factors: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["exponents", b"exponents", "factors", b"factors"]) -> None: ...

global___SpikingArrivalConfigDTO = SpikingArrivalConfigDTO

@typing_extensions.final
class PieceWiseConstantArrivalConfigDTO(google.protobuf.message.Message):
    """PieceWiseConstantArrivalConfigDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BREAKVALUES_FIELD_NUMBER: builtins.int
    BREAKPOINTS_FIELD_NUMBER: builtins.int
    @property
    def breakvalues(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def breakpoints(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        breakvalues: collections.abc.Iterable[builtins.float] | None = ...,
        breakpoints: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["breakpoints", b"breakpoints", "breakvalues", b"breakvalues"]) -> None: ...

global___PieceWiseConstantArrivalConfigDTO = PieceWiseConstantArrivalConfigDTO

@typing_extensions.final
class EPTMPArrivalConfigDTO(google.protobuf.message.Message):
    """EPTMPArrivalConfigDTO"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    THETAS_FIELD_NUMBER: builtins.int
    GAMMAS_FIELD_NUMBER: builtins.int
    PHIS_FIELD_NUMBER: builtins.int
    OMEGAS_FIELD_NUMBER: builtins.int
    @property
    def thetas(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def gammas(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def phis(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def omegas(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        thetas: collections.abc.Iterable[builtins.float] | None = ...,
        gammas: collections.abc.Iterable[builtins.float] | None = ...,
        phis: collections.abc.Iterable[builtins.float] | None = ...,
        omegas: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["gammas", b"gammas", "omegas", b"omegas", "phis", b"phis", "thetas", b"thetas"]) -> None: ...

global___EPTMPArrivalConfigDTO = EPTMPArrivalConfigDTO



from csle_collector.docker_stats_manager.dao.docker_stats import DockerStats


class TestDockerStatsManagerDaoSuite:
    """
    Test suite for datasets data access objects (DAOs) in docker_stats_manager
    """

    def test_docker_stats(self, example_docker_stats: DockerStats) -> None:
        """
        Tests creation and dict conversion of the DockerStats DAO

        :param example_docker_stats: an example DockerStats
        :return: None
        """
        assert isinstance(example_docker_stats.to_dict(), dict)
        assert isinstance(DockerStats.from_dict(example_docker_stats.to_dict()), DockerStats)
        assert (DockerStats.from_dict(example_docker_stats.to_dict()).to_dict() == example_docker_stats.to_dict())
        assert (DockerStats.from_dict(example_docker_stats.to_dict()) == example_docker_stats)
